"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.dateDataSets = void 0;
var lodash_1 = require("lodash");
var HttpStatusCodes = require("http-status-codes");
var filtering_1 = require("../helpers/filtering");
var metadataObjects_1 = require("../helpers/metadataObjects");
var groups_1 = require("../helpers/groups");
var ValidationError_1 = require("../ValidationError");
var dateDataSetsErrors = require("../../constants/DateDataSetsErrors");
function validate(request) {
    if (Object.keys(request).length !== 1) {
        throw new ValidationError_1.ValidationError(dateDataSetsErrors.WRONG_STRUCTURE);
    }
    if (!lodash_1.has(request, "dateDataSetsRequest")) {
        throw new ValidationError_1.ValidationError(dateDataSetsErrors.STRUCTURE_NOT_DEFINED);
    }
    if (!lodash_1.isPlainObject(request.dateDataSetsRequest)) {
        throw new ValidationError_1.ValidationError(dateDataSetsErrors.INVALID_STRUCTURE);
    }
}
function findMockedResponse(project, request) {
    if (!lodash_1.has(project, "endPoints.loadDateDatasetsMocks")) {
        return null;
    }
    var loadDateDatasetsMocks = project.endPoints.loadDateDatasetsMocks;
    var matchedMock = loadDateDatasetsMocks.find(function (mock) { return lodash_1.isEqual(mock.request, request); });
    return !lodash_1.isEmpty(matchedMock) ? matchedMock.response : null;
}
function filterDateDataSets(project, request) {
    var requestSelector = request.dateDataSetsRequest.requiredDataSets;
    var items = lodash_1.flatMap(groups_1.getGroups(requestSelector, project), function (group) { return group.dateDataSets; });
    return metadataObjects_1.filterByTags(request.dateDataSetsRequest, items);
}
function generateResponse(project, request) {
    var dateDataSetsItems = filterDateDataSets(project, request);
    var requestGranularities = lodash_1.get(request, "dateDataSetsRequest.includeDateGranularities", []);
    var granularities = __spreadArrays([
        "GDC.time.date",
        "GDC.time.month",
        "GDC.time.quarter",
        "GDC.time.week_us",
        "GDC.time.year"
    ], requestGranularities);
    var dateDataSetsItemsFiltered = filtering_1.filterAvailable(project, request.dateDataSetsRequest.bucketItems, dateDataSetsItems);
    var dateDataSetsItemsFilteredByGranularities = dateDataSetsItemsFiltered.map(function (dateDataset) { return (__assign(__assign({}, dateDataset), { availableDateAttributes: dateDataset.availableDateAttributes.filter(function (attribute) {
            return granularities.includes(attribute.type);
        }) })); });
    return {
        dateDataSetsResponse: {
            dateDataSets: dateDataSetsItemsFilteredByGranularities.map(metadataObjects_1.getWrappedMetadataObject),
            unavailableDateDataSetsCount: dateDataSetsItems.length - dateDataSetsItemsFilteredByGranularities.length
        }
    };
}
exports.dateDataSets = {
    register: function (app, project) {
        app.post("/gdc/internal/projects/" + project.project.meta.identifier + "/loadDateDataSets", function (req, res) {
            var request = req.body;
            try {
                validate(request);
                var mockedResponse = findMockedResponse(project, request);
                var body = mockedResponse !== null ? mockedResponse : generateResponse(project, request);
                res.status(HttpStatusCodes.OK).json(body);
            }
            catch (error) {
                if (error instanceof ValidationError_1.ValidationError) {
                    res.status(HttpStatusCodes.BAD_REQUEST).json(error.getValidationError());
                }
                else {
                    res.status(HttpStatusCodes.INTERNAL_SERVER_ERROR).json(error);
                }
            }
        });
        return app;
    }
};
//# sourceMappingURL=dateDataSets.js.map