"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.objects = void 0;
var paging_1 = require("../helpers/paging");
var filtering_1 = require("../helpers/filtering");
var metadataObjects_1 = require("../helpers/metadataObjects");
function getQueriedObjects(project, category, author, title) {
    var selectors = {
        visualizationObject: project.visualizationObjects,
        kpiAlert: project.kpiAlerts,
        visualizationClass: project.visualizationClasses,
        dateFilterConfig: [project.dateFilterConfig]
    };
    var filteredByAuthor = filtering_1.filterByAuthor(author, metadataObjects_1.getAuthor, selectors[category]);
    return filtering_1.filterByTitle(title, metadataObjects_1.getTitle, filteredByAuthor);
}
var KNOWN_CATEGORIES = ["visualizationobject", "kpialert", "visualizationclass", "datefilterconfig"]; // in lowercase !!!
function isUnknownCategory(category) {
    return KNOWN_CATEGORIES.indexOf(category.toLowerCase()) === -1;
}
function validate(req) {
    var category = req.query.category;
    if (isUnknownCategory(category)) {
        throw new Error("Uknown category");
    }
}
function isNextLinkRequired(items, offset, limit) {
    return items.slice(offset + limit).length > 0;
}
function getNextLink(req, offset, limit) {
    var queryParams = __assign(__assign({}, req.query), { offset: offset + limit });
    var queryString = Object.keys(queryParams)
        .map(function (key) { return key + "=" + encodeURIComponent(queryParams[key]); })
        .join("&");
    return req.path + "?" + queryString;
}
exports.objects = {
    register: function (app, project) {
        app.get("/gdc/md/" + project.project.meta.identifier + "/objects/query", function (req, res) {
            try {
                validate(req);
                var offset = Number(req.query.offset) || 0;
                var limit = Number(req.query.limit) || 50;
                var items = getQueriedObjects(project, req.query.category, req.query.author, req.query.title);
                var paged = paging_1.getPage(items, offset, limit);
                var body = {
                    objects: {
                        items: paged.map(metadataObjects_1.getWrappedMetadataObject),
                        paging: {
                            offset: offset,
                            count: paged.length
                        }
                    }
                };
                if (req.query.getTotalCount) {
                    body.objects.paging.totalCount = items.length;
                }
                if (isNextLinkRequired(items, offset, limit)) {
                    body.objects.paging.next = getNextLink(req, offset, limit);
                }
                res.status(200).json(body);
            }
            catch (_a) {
                res.status(404).end();
            }
        });
        return app;
    }
};
//# sourceMappingURL=objects.js.map