"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.organizedCatalog = void 0;
var lodash_1 = require("lodash");
var mockRequest_1 = require("../helpers/mockRequest");
var HttpStatusCodes = require("http-status-codes");
var groups_1 = require("../helpers/groups");
var Attribute_1 = require("../../model/Attribute");
var Metric_1 = require("../../model/Metric");
exports.organizedCatalog = {
    register: function (app, project) {
        mockCatalogGroups(app, project);
        mockCatalogQuery(app, project);
        mockCatalogItems(app, project);
        return app;
    }
};
var mockCatalogGroups = function (app, project) {
    var organizedCatalog = project.organizedCatalog ? project.organizedCatalog : {};
    var groups = organizedCatalog.groups ? organizedCatalog.groups : [];
    var projectId = project.project.meta.identifier;
    var defaultGroups = {
        response: {
            catalogGroups: []
        }
    };
    mockRequest_1.mock(app, "get", "/gdc/internal/projects/" + projectId + "/catalog/groups", groups, defaultGroups);
};
var mockCatalogQuery = function (app, project) {
    var projectId = project.project.meta.identifier;
    var organizedCatalog = project.organizedCatalog ? project.organizedCatalog : {};
    var queryResponse = organizedCatalog.queryResponse ? organizedCatalog.queryResponse : [];
    app.post("/gdc/internal/projects/" + projectId + "/catalog/query", function (req, res) {
        var mockedResponse = mockRequest_1.getMatchingResponse(req, queryResponse);
        if (mockedResponse) {
            res.status(HttpStatusCodes.OK).json(mockedResponse.response);
            return;
        }
        var requestedTypes = req.body.catalogQueryRequest.types;
        var generatedResponse = generateCatalogQueryResponse(requestedTypes, project);
        res.status(HttpStatusCodes.OK).json(generatedResponse);
    });
};
var generateCatalogQueryResponse = function (requestedTypes, project) {
    var filteredGroupItems = filterGroupItems(requestedTypes, project);
    return {
        catalogAvailableItems: {
            items: filteredGroupItems.map(function (groupItem) { return groupItem.meta.uri; })
        }
    };
};
var mockCatalogItems = function (app, project) {
    var projectId = project.project.meta.identifier;
    var organizedCatalog = project.organizedCatalog ? project.organizedCatalog : {};
    var items = organizedCatalog.items ? organizedCatalog.items : [];
    app.get("/gdc/internal/projects/" + projectId + "/catalog/items", function (req, res) {
        var mockedResponse = mockRequest_1.getMatchingResponse(req, items);
        if (mockedResponse) {
            res.status(HttpStatusCodes.OK).json(mockedResponse.response);
            return;
        }
        var requestPayload = getCatalogItemsRequest(req);
        var generatedResponse = generateCatalogItemsResponse(requestPayload, project);
        res.status(HttpStatusCodes.OK).json(generatedResponse);
    });
};
var getCatalogItemsRequest = function (req) {
    var _a = req.query, _b = _a.offset, offset = _b === void 0 ? "0" : _b, _c = _a.limit, limit = _c === void 0 ? "500" : _c, types = _a.types, production = _a.production;
    return {
        offset: Number(offset),
        limit: Number(limit),
        types: types,
        production: Number(production)
    };
};
var generateCatalogItemsResponse = function (requestPayload, project) {
    var catalogItems = generateCatalogItems(requestPayload, project);
    return {
        catalogItems: {
            items: catalogItems,
            paging: { count: catalogItems.length, offset: requestPayload.offset }
        }
    };
};
var generateCatalogItems = function (requestPayload, project) {
    var requestedTypes = requestPayload.types ? [requestPayload.types] : undefined;
    var filteredCatalogItems = filterGroupItems(requestedTypes, project);
    return filteredCatalogItems.map(generateCatalogItem);
};
var generateCatalogItem = function (groupItem) {
    if (Attribute_1.isAttribute(groupItem)) {
        return {
            type: "attribute",
            identifier: groupItem.meta.identifier,
            title: groupItem.meta.title,
            summary: groupItem.meta.summary,
            links: {
                self: groupItem.meta.uri,
                defaultDisplayForm: groupItem.content.displayForms.length > 0
                    ? groupItem.content.displayForms[0].meta.uri
                    : undefined
            }
        };
    }
    else if (Metric_1.isMetric(groupItem)) {
        return {
            type: "metric",
            identifier: groupItem.meta.identifier,
            title: groupItem.meta.title,
            expression: groupItem.content.expression,
            summary: groupItem.meta.summary,
            links: {
                self: groupItem.meta.uri
            }
        };
    }
    else {
        return {
            type: "fact",
            identifier: groupItem.meta.identifier,
            title: groupItem.meta.title,
            summary: groupItem.meta.summary,
            links: {
                self: groupItem.meta.uri
            }
        };
    }
};
var filterGroupItems = function (requestedTypes, project) {
    if (requestedTypes === void 0) { requestedTypes = ["attribute", "fact", "metric"]; }
    var groups = groups_1.getGroups({ type: "PRODUCTION" }, project);
    var groupItems = lodash_1.flatMap(groups, function (group) { return __spreadArrays(group.metrics, group.facts, group.attributes); });
    return groupItems.filter(function (item) { return requestedTypes.some(function (type) { return item.meta.category === type; }); });
};
//# sourceMappingURL=organized_catalog.js.map