"use strict";
exports.__esModule = true;
exports.profile = exports.CURRENT_USER_PSEUDO_ID = exports.createError = exports.createAccountSettings = void 0;
var HttpStatusCodes = require("http-status-codes");
var routes_1 = require("../../route/routes");
function createAccountSettings(user) {
    return {
        accountSetting: {
            companyName: user.companyName,
            firstName: user.firstName,
            lastName: user.lastName,
            phoneNumber: user.phoneNumber,
            login: user.email,
            separators: user.separators,
            email: user.email,
            authenticationModes: [],
            effectiveIpWhitelist: null,
            ipWhitelist: null,
            country: null,
            language: "en-US",
            position: null,
            timezone: null,
            links: {
                self: routes_1.profileUri(user.identifier)
            },
            created: "2014-07-01 15:16:05",
            updated: "2017-01-03 14:15:39"
        }
    };
}
exports.createAccountSettings = createAccountSettings;
function createError(userId) {
    return {
        error: {
            parameters: [userId],
            component: "GDC::LWP::UserAgent",
            errorClass: "GDC::Exception::NotFound",
            message: "User uid %s doesn't exist."
        }
    };
}
exports.createError = createError;
exports.CURRENT_USER_PSEUDO_ID = "current";
exports.profile = {
    register: function (app, project) {
        app.get("/gdc/account/profile/:id", function (req, res) {
            var id = req.params.id;
            var user = id === exports.CURRENT_USER_PSEUDO_ID
                ? project.users[0]
                : project.users.find(function (u) { return u.identifier === id; });
            if (user) {
                res.status(HttpStatusCodes.OK).json(createAccountSettings(user));
            }
            else {
                res.status(HttpStatusCodes.NOT_FOUND).json(createError(id));
            }
        });
        app.put("/gdc/account/profile/:id/settings", function (req, res) {
            var id = req.params.id;
            var user = id === exports.CURRENT_USER_PSEUDO_ID
                ? project.users[0]
                : project.users.find(function (u) { return u.identifier === id; });
            if (user) {
                res.status(HttpStatusCodes.NO_CONTENT).end();
            }
            else {
                res.status(HttpStatusCodes.NOT_FOUND).json(createError(id));
            }
        });
        return app;
    }
};
//# sourceMappingURL=profile.js.map