"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.query = void 0;
var es6_promise_1 = require("es6-promise");
var MockProject_1 = require("../../model/MockProject");
var HttpStatusCodes = require("http-status-codes");
var PENDING_IMPLEMENTATIONS = ["projectdashboards"];
var KNOWN_TYPES = __spreadArrays(["attributes", "analyticaldashboard"], PENDING_IMPLEMENTATIONS);
function getAnalyticalDashboards(project) {
    return project.analyticalDashboards;
}
function queryObjects(project, type) {
    if (isUnknownType(type)) {
        return es6_promise_1.Promise.reject(createError(type));
    }
    if (PENDING_IMPLEMENTATIONS.indexOf(type) >= 0) {
        return es6_promise_1.Promise.resolve([]);
    }
    // TODO: We need more generic system to get different kinds of metadata objects
    if (type === "analyticaldashboard") {
        var links = getAnalyticalDashboards(project).map(function (dashboard) { return createLink(dashboard.meta); });
        return es6_promise_1.Promise.resolve(links);
    }
    return es6_promise_1.Promise.resolve(MockProject_1.getAttributes(project).map(function (attribute) { return createLink(attribute.meta); }));
}
function createLink(meta) {
    var link = __assign(__assign({}, meta), { link: meta.uri });
    delete link.uri;
    return link;
}
function isUnknownType(objectType) {
    return KNOWN_TYPES.indexOf(objectType) === -1;
}
function createError(objectType) {
    return {
        error: {
            component: "metadata",
            errorClass: "foo.bar",
            message: "Unknown type '%s'",
            parameters: [objectType]
        }
    };
}
function createQuery(entries, project, type) {
    return {
        query: {
            entries: entries,
            meta: {
                category: "query",
                summary: "Meta Query Resources for project '" + project + "'",
                title: "List of " + type
            }
        }
    };
}
exports.query = {
    register: function (app, project) {
        app.get("/gdc/md/" + project.project.meta.identifier + "/query/:type", function (req, res) {
            queryObjects(project, req.params.type)
                .then(function (entries) { return [
                HttpStatusCodes.OK,
                createQuery(entries, project.project.meta.identifier, req.params.type),
            ]; })["catch"](function (error) { return [HttpStatusCodes.NOT_FOUND, error]; })
                .then(function (_a) {
                var status = _a[0], body = _a[1];
                return res.status(status).json(body);
            });
        });
        return app;
    }
};
//# sourceMappingURL=query.js.map