"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.using2 = void 0;
// (C) 2007-2019 GoodData Corporation
var errors = require("../errors/errors");
var HttpStatusCodes = require("http-status-codes");
var lodash_1 = require("lodash");
var MockProject_1 = require("../../model/MockProject");
var ValidationError_1 = require("../ValidationError");
var Widget_1 = require("../../model/Widget");
function validate(request) {
    if (!lodash_1.has(request, "inUse") && !lodash_1.has(request, "inUseMany")) {
        // inUseMix isn't implemented because it's not used in gdc-dashboards
        throw new ValidationError_1.ValidationError(errors.undefinedUsing2Structure(Object.keys(request)[0]));
    }
}
function getVisualizationUriFromVisualizationWidget(visualizationWidget) {
    return visualizationWidget.content.visualization;
}
function getInUseBody(request, project) {
    var dashboardUri = request.inUse.uri;
    var dashboard = project.analyticalDashboards.find(function (d) { return d.meta.uri === dashboardUri; });
    var widgets = __spreadArrays(MockProject_1.getFilterContexts(project), MockProject_1.getVisualizationObjects(project), MockProject_1.getWidgets(project));
    var visualizationWidgets = MockProject_1.getVisualizationWidgets(project);
    var filterContextUri = dashboard.content.filterContext;
    var visualizationsUris = visualizationWidgets.map(function (visualizationWidget) {
        return getVisualizationUriFromVisualizationWidget(visualizationWidget);
    });
    var visualizationWidgetsUris = dashboard.content.widgets;
    var widgetUris = __spreadArrays([filterContextUri], visualizationsUris, visualizationWidgetsUris);
    var entries = widgets
        .filter(function (widget) {
        var widgetUri = Widget_1.getWidgetUri(widget);
        return lodash_1.includes(widgetUris, widgetUri);
    })
        .filter(function (widget) { return request.inUse.types.some(function (type) { return type === Widget_1.getWidgetCategory(widget); }); })
        .map(function (widget) { return objectMetaToLinkResponse(widget.meta); });
    return { entries: entries };
}
function objectMetaToLinkResponse(objectMeta) {
    return Object.keys(lodash_1.omit(objectMeta, ["uri"])).reduce(function (linkResponse, key) {
        if (key === "deprecated") {
            linkResponse.deprecated = objectMeta.deprecated ? "1" : "0";
        }
        else if (lodash_1.includes(["isProduction", "sharedWithSomeone", "unlisted", "locked"], key)) {
            linkResponse[key] = objectMeta[key] ? 1 : 0;
        }
        else if (key !== "locked") {
            linkResponse[key] = objectMeta[key];
        }
        return linkResponse;
    }, { link: objectMeta.uri });
}
function getInUseManyBody(request, project) {
    var requestedDisplayFormUris = request.inUseMany.uris;
    if (!lodash_1.includes(request.inUseMany.types, "attribute")) {
        return { useMany: [] };
    }
    var useManyResponse = MockProject_1.getAttributes(project).reduce(function (result, attribute) {
        var displayForm = attribute.content.displayForms.find(function (df) {
            var displayFormUri = df.meta.uri;
            return lodash_1.includes(requestedDisplayFormUris, displayFormUri);
        });
        if (displayForm) {
            var attributeObjectMeta = attribute.meta;
            var entryResponse = objectMetaToLinkResponse(attributeObjectMeta);
            return __spreadArrays(result, [{ entries: [entryResponse], uri: displayForm.meta.uri }]);
        }
        return result;
    }, []);
    return { useMany: useManyResponse };
}
exports.using2 = {
    register: function (app, project) {
        app.post("/gdc/md/" + project.project.meta.identifier + "/using2", function (req, res) {
            try {
                validate(req.body);
                if (lodash_1.has(req.body, "inUse")) {
                    res.status(HttpStatusCodes.OK).json(getInUseBody(req.body, project));
                }
                else {
                    res.status(HttpStatusCodes.OK).json(getInUseManyBody(req.body, project));
                }
            }
            catch (error) {
                if (error instanceof ValidationError_1.ValidationError) {
                    res.status(HttpStatusCodes.BAD_REQUEST).json(error.getValidationError());
                }
                else {
                    res.status(HttpStatusCodes.INTERNAL_SERVER_ERROR).json(error);
                }
            }
        });
        return app;
    }
};
//# sourceMappingURL=using2.js.map