"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.validElements = void 0;
// (C) 2007-2021 GoodData Corporation
var lodash_1 = require("lodash");
var MockProject_1 = require("../../model/MockProject");
var routes_1 = require("../../route/routes");
var HttpStatusCodes = require("http-status-codes");
var ordering_1 = require("../helpers/ordering");
var filtering_1 = require("../helpers/filtering");
var paging_1 = require("../helpers/paging");
function getItems(project, attributeId) {
    return project.elements.get(attributeId) || [];
}
function createError(attributeId) {
    return {
        error: {
            parameters: [attributeId],
            requestId: "PIeM4i3QEwcBhjvI:j7db5gbwtmwa3nac",
            component: "Resource::Elements::Service",
            errorClass: "GDC::Exception::User",
            message: "Object %s is not a display form"
        }
    };
}
function getPipeLine(items, filter, order, uris) {
    var getTitle = function (item) { return item.element.title; };
    var getUri = function (item) { return item.element.uri; };
    return lodash_1.flow(lodash_1.partial(filtering_1.filterByUri, uris, getUri), lodash_1.partial(filtering_1.filterByTitle, filter, getTitle), lodash_1.partial(ordering_1.orderByTitle, order, getTitle))(items);
}
function getAttribute(project, identifier) {
    var attributeHash = MockProject_1.getAttributes(project).reduce(function (hash, attribute) {
        var _a;
        return __assign(__assign({}, hash), (_a = {}, _a[attribute.content.displayForms[0].meta.identifier] = attribute, _a));
    }, {});
    var dateAttributeHash = MockProject_1.getDateAttributes(project).reduce(function (hash, attribute) {
        var _a;
        return __assign(__assign({}, hash), (_a = {}, _a[attribute.displayForms[0].meta.identifier] = attribute, _a));
    }, {});
    var hash = __assign(__assign({}, attributeHash), dateAttributeHash);
    return hash[identifier];
}
exports.validElements = {
    register: function (app, project) {
        app.post("/gdc/md/" + project.project.meta.identifier + "/obj/:id/validElements", function (req, res) {
            var attribute = getAttribute(project, req.params.id);
            if (attribute) {
                var requestBody = req.body;
                var projectId = project.project.meta.identifier;
                var uris = lodash_1.get(requestBody, "validElementsRequest.uris", []);
                var items = getPipeLine(getItems(project, attribute.meta.identifier), req.query.filter, req.query.order, uris);
                var _a = req.query, _b = _a.offset, offset = _b === void 0 ? 0 : _b, limit = _a.limit;
                var sliced = paging_1.getPage(items, Number(offset), Number(limit));
                var body = {
                    validElements: {
                        items: sliced,
                        paging: {
                            total: items.length,
                            count: sliced.length,
                            offset: Number(offset)
                        },
                        elementsMeta: {
                            attribute: routes_1.objectUri(projectId, attribute.meta.identifier),
                            attributeDisplayForm: routes_1.objectUri(projectId, req.params.id),
                            order: (req.query.order || "asc"),
                            filter: (req.query.filter || "")
                        }
                    }
                };
                res.status(HttpStatusCodes.OK).json(body);
            }
            else {
                res.status(HttpStatusCodes.BAD_REQUEST).json(createError(req.params.id));
            }
        });
        return app;
    }
};
//# sourceMappingURL=validElements.js.map