"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.filterAvailable = exports.filterByUri = exports.filterByAuthor = exports.filterByTitle = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var metadataObjects_1 = require("./metadataObjects");
var maql_1 = require("./maql");
function filterByTitle(filter, getTitle, items) {
    if (filter === void 0) { filter = ""; }
    return filter
        ? items.filter(function (item) {
            return getTitle(item)
                .toLowerCase()
                .includes(filter.toLowerCase());
        })
        : items;
}
exports.filterByTitle = filterByTitle;
function filterByAuthor(author, getAuthor, items) {
    if (author === void 0) { author = ""; }
    return author ? items.filter(function (item) { return getAuthor(item) === author; }) : items;
}
exports.filterByAuthor = filterByAuthor;
function filterByUri(uris, getUri, items) {
    return uris.length ? items.filter(function (item) { return uris.indexOf(getUri(item)) >= 0; }) : items;
}
exports.filterByUri = filterByUri;
function filterAvailable(project, bucketItems, items) {
    if (bucketItems === void 0) { bucketItems = []; }
    function getGroup(uri) {
        return project.groups.findIndex(function (group) {
            var attributes = group.attributes, metrics = group.metrics, facts = group.facts, dateDataSets = group.dateDataSets;
            var dateAttributes = lodash_1.flatMap(dateDataSets, function (dateDataSet) { return dateDataSet.availableDateAttributes; });
            return __spreadArrays(attributes, metrics, facts, dateDataSets, dateAttributes).map(metadataObjects_1.getUri)
                .some(lodash_1.partial(lodash_1.isEqual, uri));
        });
    }
    function isRelated(extendedBucketItems, item) {
        return extendedBucketItems.every(function (bucketItem) { return getGroup(bucketItem) === getGroup(metadataObjects_1.getUri(item)); });
    }
    var isRelatedToBuckets = lodash_1.partial(isRelated, maql_1.reduceMAQLItems(bucketItems));
    return bucketItems.length ? items.filter(isRelatedToBuckets) : items;
}
exports.filterAvailable = filterAvailable;
//# sourceMappingURL=filtering.js.map