"use strict";
exports.__esModule = true;
exports.getWrappedMetadataObject = exports.filterByTags = exports.filterItem = exports.parseTags = exports.getAuthor = exports.getTags = exports.getUri = exports.getTitle = exports.getIdentifier = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var VisualizationClass_1 = require("../../model/VisualizationClass");
var VisualizationObject_1 = require("../../model/VisualizationObject");
var VisualizationWidget_1 = require("../../model/VisualizationWidget");
var Kpi_1 = require("../../model/Kpi");
var FilterContext_1 = require("../../model/FilterContext");
var AttributeDisplayForm_1 = require("../../model/AttributeDisplayForm");
var Fact_1 = require("../../model/Fact");
var Metric_1 = require("../../model/Metric");
var Attribute_1 = require("../../model/Attribute");
var KpiAlert_1 = require("../../model/KpiAlert");
var DateDataSet_1 = require("../../model/DateDataSet");
var AnalyticalDashboard_1 = require("../../model/AnalyticalDashboard");
var DateFilterConfig_1 = require("../../model/DateFilterConfig");
function getIdentifier(item) {
    return item.meta.identifier;
}
exports.getIdentifier = getIdentifier;
function getTitle(item) {
    return item.meta.title;
}
exports.getTitle = getTitle;
function getUri(item) {
    return item.meta.uri;
}
exports.getUri = getUri;
function getTags(item) {
    return item.meta.tags;
}
exports.getTags = getTags;
function getAuthor(item) {
    return item.meta.author;
}
exports.getAuthor = getAuthor;
function parseTags(item) {
    var tags = getTags(item);
    return !tags ? [] : tags.split(" ");
}
exports.parseTags = parseTags;
function filterItem(filter, item) {
    var includeObjectsWithTags = filter.includeObjectsWithTags, excludeObjectsWithTags = filter.excludeObjectsWithTags;
    var tags = parseTags(item);
    if (includeObjectsWithTags) {
        return lodash_1.some(tags, lodash_1.partial(lodash_1.includes, includeObjectsWithTags));
    }
    return lodash_1.every(tags, lodash_1.negate(lodash_1.partial(lodash_1.includes, excludeObjectsWithTags)));
}
exports.filterItem = filterItem;
function filterByTags(request, items) {
    return items.filter(lodash_1.partial(filterItem, request));
}
exports.filterByTags = filterByTags;
function getWrappedMetadataObject(object) {
    if (VisualizationObject_1.isVisualizationObject(object)) {
        return { visualizationObject: object };
    }
    if (VisualizationClass_1.isVisualizationClass(object)) {
        return { visualizationClass: object };
    }
    if (VisualizationWidget_1.isVisualizationWidget(object)) {
        return { visualizationWidget: object };
    }
    if (Kpi_1.isKpi(object)) {
        return { kpi: object };
    }
    if (FilterContext_1.isFilterContext(object)) {
        return { filterContext: object };
    }
    if (AttributeDisplayForm_1.isAttributeDisplayForm(object)) {
        return { attributeDisplayForm: object };
    }
    if (Fact_1.isFact(object)) {
        return { fact: object };
    }
    if (Metric_1.isMetric(object)) {
        return { metric: object };
    }
    // needs to be before isAttribute
    if (DateDataSet_1.isDateAttribute(object)) {
        return DateDataSet_1.wrapDateAttribute(object);
    }
    if (Attribute_1.isAttribute(object)) {
        return { attribute: object };
    }
    if (KpiAlert_1.isKpiAlert(object)) {
        return { kpiAlert: object };
    }
    if (DateDataSet_1.isDateDataSet(object)) {
        return DateDataSet_1.wrapDateDataSet(object);
    }
    if (AnalyticalDashboard_1.isAnalyticalDashboard(object)) {
        return { analyticalDashboard: object };
    }
    if (DateFilterConfig_1.isDateFilterConfig(object)) {
        return { dateFilterConfig: object };
    }
    console.log("unknown object: ", object); // tslint:disable-line:no-console
}
exports.getWrappedMetadataObject = getWrappedMetadataObject;
//# sourceMappingURL=metadataObjects.js.map