"use strict";
exports.__esModule = true;
exports.mock = exports.getMatchingResponse = exports.isMockRequestMatching = void 0;
var lodash_1 = require("lodash");
var HttpStatusCodes = require("http-status-codes");
function isRequestHeadersMatching(headers, config) {
    if (!config.headers) {
        return true;
    }
    if (!headers) {
        return false;
    }
    return Object.keys(config.headers).every(function (key) { return config.headers[key] === headers[key]; });
}
function isMockRequestMatching(req, config) {
    var keysMatch = ["params", "query", "body"].every(function (key) { return lodash_1.isEqual(req[key] || {}, config[key] || {}); });
    var headersMatch = isRequestHeadersMatching(req.headers, config);
    return keysMatch && headersMatch;
}
exports.isMockRequestMatching = isMockRequestMatching;
function getMatchingResponse(req, mocks) {
    if (mocks === void 0) { mocks = []; }
    return mocks.find(function (mock) { return isMockRequestMatching(req, mock); });
}
exports.getMatchingResponse = getMatchingResponse;
function mock(app, method, uri, mockConfigs, defaultMockConfig) {
    if (defaultMockConfig === void 0) { defaultMockConfig = null; }
    app[method](uri, function (req, res) {
        var mockedResponseConfig = getMatchingResponse(req, mockConfigs) || defaultMockConfig;
        if (mockedResponseConfig) {
            res.status(HttpStatusCodes.OK).json(mockedResponseConfig.response);
            return;
        }
        res.status(HttpStatusCodes.BAD_REQUEST).send("Can't find mock for this request");
    });
}
exports.mock = mock;
//# sourceMappingURL=mockRequest.js.map