"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.register = void 0;
// (C) 2007-2019 GoodData Corporation
var fs = require("fs");
var lodash_1 = require("lodash");
var bodyParser = require("body-parser");
var session = require("express-session");
var builder_1 = require("../schema/builder");
var bear = require("./bear");
var webapp = require("./webapp");
var mockBuilder = require("./utils");
var tokenExpiration_1 = require("../middleware/tokenExpiration");
var defaultConfig = {
    randomSeed: null,
    pollCount: 5
};
var getSuffix = function (str, delimiter) { return str.split(delimiter).pop(); };
var getPathsToLoadedModulesWithSuffix = function (suffix, delimiter) {
    return Object.keys(require.cache).filter(function (path) { return getSuffix(path, delimiter) === suffix; });
};
function reloadModule(module) {
    var submodules = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        submodules[_i - 1] = arguments[_i];
    }
    submodules.forEach(function (submodule) {
        delete require.cache[require.resolve(submodule)];
    });
    delete require.cache[require.resolve(module)];
    return require(module);
}
function register(app, options) {
    app.use(bodyParser.json({ limit: "50mb" }));
    app.use(session({
        secret: "#sorryjako",
        resave: false,
        saveUninitialized: false
    }));
    if (options.config && options.config.useTokenExpiration) {
        app.use(tokenExpiration_1.tokenExpiration({
            expirationTime: 10000,
            publicPaths: ["/gdc/account/login"]
        }));
    }
    var projects = [];
    options.schema.forEach(function (schema) {
        var endpoints = __assign(__assign(__assign({}, bear), webapp), mockBuilder);
        var config = __assign(__assign({}, defaultConfig), options.config);
        var project = builder_1.buildProject(schema);
        projects.push(project);
        lodash_1.values(endpoints)
            .filter(lodash_1.isPlainObject)
            .forEach(function (endpoint) { return endpoint.register(app, project, config); });
    });
    if (options.absolutePathToRootMockFile && options.getSchema) {
        var pathsToMockFiles = options.suffixOfMockFilesImportedInRootMockFile
            ? getPathsToLoadedModulesWithSuffix(options.suffixOfMockFilesImportedInRootMockFile, "_")
            : [];
        pathsToMockFiles.push(options.absolutePathToRootMockFile);
        pathsToMockFiles.forEach(function (pathToMockFile) {
            fs.watch(pathToMockFile, { encoding: "buffer" }, function (_, fileName) {
                var newOptions = reloadModule(options.absolutePathToRootMockFile, pathToMockFile);
                var newProjects = options.getSchema(newOptions).map(builder_1.buildProject);
                lodash_1.merge(projects, newProjects);
                // tslint:disable-next-line:no-console
                console.log("Mockfile '" + fileName + "' was reloaded.");
            });
        });
    }
    return app;
}
exports.register = register;
//# sourceMappingURL=register.js.map