"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.mockBuilder = void 0;
var lodash_1 = require("lodash");
var http_status_codes_1 = require("http-status-codes");
var typings_1 = require("@gooddata/typings");
var mockBuilderUi_1 = require("./mockBuilderUi");
var execution_1 = require("../webapp/executeAfm/execution");
var isAttributeHeader = typings_1.Execution.isAttributeHeader;
var isMeasureGroupHeader = typings_1.Execution.isMeasureGroupHeader;
var isPositiveAttributeFilter = typings_1.AFM.isPositiveAttributeFilter;
var isNegativeAttributeFilter = typings_1.AFM.isNegativeAttributeFilter;
var isPopMeasureDefinition = typings_1.AFM.isPopMeasureDefinition;
var isPreviousPeriodMeasureDefinition = typings_1.AFM.isPreviousPeriodMeasureDefinition;
var isSimpleMeasureDefinition = typings_1.AFM.isSimpleMeasureDefinition;
var SchemaDictionary = /** @class */ (function () {
    function SchemaDictionary(project, projectGroupIndex) {
        var _this = this;
        this.schemaAttributeElementsMapping = {}; // schema attr uri to attr elements
        this.attributeMapping = {}; // staging attr uri to schema attr uri
        this.metadataObjectCache = {}; // staging metadata obj uri to schema metadata obj uri
        this.schemaAttributeIndex = 0;
        this.schemaMetricIndex = 0;
        this.schemaDateDataSetIndex = 0;
        var schemaDataSet = project.groups[projectGroupIndex];
        this.projectId = project.project.meta.identifier;
        this.schemaDataSet = schemaDataSet;
        this.schemaAttributesWithElements = schemaDataSet.attributes.filter(function (attribute) {
            var meta = attribute.meta;
            var attributeElements = project.elements.get(meta.identifier);
            _this.schemaAttributeElementsMapping[meta.uri] = {
                elements: attributeElements,
                index: 0
            };
            return attributeElements !== undefined && attributeElements.length > 0;
        });
    }
    SchemaDictionary.prototype.getAttributeDisplayFormObjQualifier = function (displayFormObjIdentifier) {
        var attribute = this.getNextSchemaAttribute(this.extractUri(displayFormObjIdentifier));
        return this.asUriObjQualifier(attribute.content.displayForms[0]);
    };
    SchemaDictionary.prototype.getAttribute = function (displayFormObjIdentifier, attributeUri) {
        var attribute = this.getNextSchemaAttribute(this.extractUri(displayFormObjIdentifier));
        this.attributeMapping[attributeUri] = attribute.meta.uri;
        return attribute;
    };
    SchemaDictionary.prototype.getAttributeElementUri = function (elementUri, schemaAttributeUri) {
        var cachedElementUri = this.metadataObjectCache[elementUri];
        if (cachedElementUri !== undefined) {
            return cachedElementUri;
        }
        var attributeUri = elementUri.substr(0, elementUri.indexOf("/elements?id="));
        var elementMappingUri = schemaAttributeUri === undefined ? this.attributeMapping[attributeUri] : schemaAttributeUri;
        var attributeElements = this.schemaAttributeElementsMapping[elementMappingUri];
        var nextElement = attributeElements.elements[attributeElements.index++];
        if (nextElement === undefined) {
            throw new Error("There are no more elements in " + elementMappingUri + "!");
        }
        var nextElementUri = nextElement.element.uri;
        this.metadataObjectCache[elementUri] = nextElementUri;
        return nextElementUri;
    };
    SchemaDictionary.prototype.getMeasureObjQualifier = function (objIdentifier) {
        var metric = this.getNextSchemaMetric(this.extractUri(objIdentifier));
        return this.asUriObjQualifier(metric);
    };
    SchemaDictionary.prototype.getMeasureMetaFor = function (measureUri) {
        var metric = this.getNextSchemaMetric(measureUri);
        return metric.meta;
    };
    SchemaDictionary.prototype.getDateDataSetObjQualifier = function (objIdentifier) {
        var dateDataSet = this.getNextSchemaDateDataSet(this.extractUri(objIdentifier));
        return this.asUriObjQualifier(dateDataSet);
    };
    SchemaDictionary.prototype.getProjectId = function () {
        return this.projectId;
    };
    SchemaDictionary.prototype.getNextSchemaAttribute = function (attributeUri) {
        var _this = this;
        return this.getMetadataObject(attributeUri, function () { return _this.schemaAttributesWithElements[_this.schemaAttributeIndex++]; });
    };
    SchemaDictionary.prototype.getNextSchemaMetric = function (measureUri) {
        var _this = this;
        return this.getMetadataObject(measureUri, function () { return _this.schemaDataSet.metrics[_this.schemaMetricIndex++]; });
    };
    SchemaDictionary.prototype.getNextSchemaDateDataSet = function (dateDataSetUri) {
        var _this = this;
        return this.getMetadataObject(dateDataSetUri, function () { return _this.schemaDataSet.dateDataSets[_this.schemaDateDataSetIndex++]; });
    };
    SchemaDictionary.prototype.extractUri = function (objIdentifier) {
        var objUriQualifier = objIdentifier;
        return objUriQualifier.uri;
    };
    SchemaDictionary.prototype.asUriObjQualifier = function (metadataObject) {
        return {
            uri: metadataObject.meta.uri
        };
    };
    SchemaDictionary.prototype.getMetadataObject = function (objUri, objectFactory) {
        var cachedObject = this.metadataObjectCache[objUri];
        if (cachedObject !== undefined) {
            return cachedObject;
        }
        var metadataObject = objectFactory();
        if (metadataObject === undefined) {
            throw new Error("There are no more objects in schema to replace " + objUri + "!");
        }
        this.metadataObjectCache[objUri] = metadataObject;
        return metadataObject;
    };
    return SchemaDictionary;
}());
var MockBuilder = /** @class */ (function () {
    function MockBuilder(project, projectGroupIndex) {
        this.schema = new SchemaDictionary(project, projectGroupIndex);
    }
    MockBuilder.prototype.buildExecutionMockWithReplacedUris = function (payload) {
        var mockedExecution = {
            execution: this.mapExecution(payload.execution),
            executionResponse: this.mapExecutionResponse(payload.executionResponse),
            executionResult: this.mapExecutionResult(payload.executionResult)
        };
        return execution_1.relativizeMockedExecutionLocalIdentifiers(mockedExecution);
    };
    MockBuilder.prototype.mapExecution = function (wrappedExecution) {
        var execution = wrappedExecution.execution, _a = wrappedExecution.execution, afm = _a.afm, _b = _a.afm, measures = _b.measures, attributes = _b.attributes, filters = _b.filters;
        return {
            execution: __assign(__assign({}, execution), { afm: __assign(__assign(__assign(__assign({}, afm), this.mapAfmMeasures(measures)), this.mapAfmAttributes(attributes)), this.mapAfmFilters(filters)) })
        };
    };
    MockBuilder.prototype.mapAfmMeasures = function (measures) {
        var _this = this;
        if (measures === undefined) {
            return {};
        }
        return {
            measures: measures.map(function (measure) { return _this.mapMeasure(measure); })
        };
    };
    MockBuilder.prototype.mapMeasure = function (measure) {
        var definition = measure.definition;
        return __assign(__assign({}, measure), { definition: __assign(__assign({}, definition), this.mapMeasureDefinition(definition)) });
    };
    MockBuilder.prototype.mapMeasureDefinition = function (definition) {
        if (isSimpleMeasureDefinition(definition)) {
            return this.mapSimpleMeasureDefinition(definition);
        }
        if (isPopMeasureDefinition(definition)) {
            return this.mapPopMeasureDefinition(definition);
        }
        if (isPreviousPeriodMeasureDefinition(definition)) {
            return this.mapPreviousPeriodMeasureDefinition(definition);
        }
        return definition;
    };
    MockBuilder.prototype.mapSimpleMeasureDefinition = function (definition) {
        var simpleMeasure = definition.measure, _a = definition.measure, filters = _a.filters, item = _a.item;
        return {
            measure: __assign(__assign(__assign({}, simpleMeasure), this.mapMeasureFilters(filters)), { item: this.schema.getMeasureObjQualifier(item) })
        };
    };
    MockBuilder.prototype.mapMeasureFilters = function (filters) {
        if (filters === undefined) {
            return {};
        }
        return {
            filters: this.mapFilters(filters)
        };
    };
    MockBuilder.prototype.mapPopMeasureDefinition = function (definition) {
        var popMeasure = definition.popMeasure;
        return {
            popMeasure: __assign(__assign({}, popMeasure), { popAttribute: this.schema.getDateDataSetObjQualifier(popMeasure.popAttribute) })
        };
    };
    MockBuilder.prototype.mapPreviousPeriodMeasureDefinition = function (definition) {
        var _this = this;
        var previousPeriodMeasure = definition.previousPeriodMeasure, dateDataSets = definition.previousPeriodMeasure.dateDataSets;
        return {
            previousPeriodMeasure: __assign(__assign({}, previousPeriodMeasure), { dateDataSets: dateDataSets.map(function (dateDataSet) {
                    return __assign(__assign({}, dateDataSet), { dataSet: _this.schema.getDateDataSetObjQualifier(dateDataSet.dataSet) });
                }) })
        };
    };
    MockBuilder.prototype.mapAfmAttributes = function (attributes) {
        var _this = this;
        if (attributes === undefined) {
            return {};
        }
        return {
            attributes: attributes.map(function (attribute) { return _this.mapAttribute(attribute); })
        };
    };
    MockBuilder.prototype.mapAttribute = function (attribute) {
        return __assign(__assign({}, attribute), { displayForm: this.schema.getAttributeDisplayFormObjQualifier(attribute.displayForm) });
    };
    MockBuilder.prototype.mapAfmFilters = function (filters) {
        if (filters === undefined) {
            return {};
        }
        return {
            filters: this.mapFilters(filters)
        };
    };
    MockBuilder.prototype.mapFilters = function (filters) {
        var _this = this;
        return filters.map(function (filter) {
            if (isPositiveAttributeFilter(filter)) {
                return _this.mapPositiveAttributeFilter(filter);
            }
            if (isNegativeAttributeFilter(filter)) {
                return _this.mapNegativeAttributeFilter(filter);
            }
            if (_this.isAbsoluteDateFilter(filter)) {
                return _this.mapAbsoluteDateFilter(filter);
            }
            if (_this.isRelativeDateFilter(filter)) {
                return _this.mapRelativeDateFilter(filter);
            }
            return filter;
        });
    };
    MockBuilder.prototype.mapPositiveAttributeFilter = function (filter) {
        var positiveAttributeFilter = filter.positiveAttributeFilter, displayForm = filter.positiveAttributeFilter.displayForm;
        var attribute = this.schema.getAttribute(displayForm);
        return __assign(__assign({}, filter), { positiveAttributeFilter: __assign(__assign({}, positiveAttributeFilter), { displayForm: {
                    uri: attribute.content.displayForms[0].meta.uri
                }, "in": this.mapAttributeElements(positiveAttributeFilter["in"], attribute.meta.uri) }) });
    };
    MockBuilder.prototype.mapAttributeElements = function (valueUris, attributeUri) {
        var _this = this;
        return valueUris.map(function (valueUri) {
            return _this.schema.getAttributeElementUri(valueUri, attributeUri);
        });
    };
    MockBuilder.prototype.mapNegativeAttributeFilter = function (filter) {
        var negativeAttributeFilter = filter.negativeAttributeFilter, _a = filter.negativeAttributeFilter, displayForm = _a.displayForm, notIn = _a.notIn;
        var attribute = this.schema.getAttribute(displayForm);
        return __assign(__assign({}, filter), { negativeAttributeFilter: __assign(__assign({}, negativeAttributeFilter), { displayForm: {
                    uri: attribute.content.displayForms[0].meta.uri
                }, notIn: this.mapAttributeElements(notIn, attribute.meta.uri) }) });
    };
    MockBuilder.prototype.isAbsoluteDateFilter = function (filter) {
        return !lodash_1.isEmpty(filter) && filter.absoluteDateFilter !== undefined;
    };
    MockBuilder.prototype.mapAbsoluteDateFilter = function (filter) {
        var absoluteDateFilter = filter.absoluteDateFilter;
        return __assign(__assign({}, filter), { absoluteDateFilter: __assign(__assign({}, absoluteDateFilter), { dataSet: this.schema.getDateDataSetObjQualifier(absoluteDateFilter.dataSet) }) });
    };
    MockBuilder.prototype.isRelativeDateFilter = function (filter) {
        return !lodash_1.isEmpty(filter) && filter.relativeDateFilter !== undefined;
    };
    MockBuilder.prototype.mapRelativeDateFilter = function (filter) {
        var relativeDateFilter = filter.relativeDateFilter;
        return __assign(__assign({}, filter), { relativeDateFilter: __assign(__assign({}, relativeDateFilter), { dataSet: this.schema.getDateDataSetObjQualifier(relativeDateFilter.dataSet) }) });
    };
    MockBuilder.prototype.mapExecutionResponse = function (wrappedExecutionResponse) {
        var _this = this;
        var executionResponse = wrappedExecutionResponse.executionResponse, _a = wrappedExecutionResponse.executionResponse, dimensions = _a.dimensions, links = _a.links;
        return {
            executionResponse: __assign(__assign({}, executionResponse), { dimensions: dimensions.map(function (dimension) { return _this.mapDimension(dimension); }), links: __assign(__assign({}, links), { executionResult: this.replaceExecutionResultProjectId(links.executionResult) }) })
        };
    };
    MockBuilder.prototype.mapDimension = function (dimension) {
        var _this = this;
        return __assign(__assign({}, dimension), { headers: dimension.headers.map(function (header) { return _this.mapResponseHeader(header); }) });
    };
    MockBuilder.prototype.mapResponseHeader = function (header) {
        if (isAttributeHeader(header)) {
            return this.mapAttributeHeader(header);
        }
        if (isMeasureGroupHeader(header)) {
            return this.mapMeasureGroupHeader(header);
        }
        return header;
    };
    MockBuilder.prototype.mapAttributeHeader = function (header) {
        var headerContent = header.attributeHeader;
        var attributeUriObjQualifier = { uri: headerContent.uri };
        var attribute = this.schema.getAttribute(attributeUriObjQualifier, headerContent.formOf.uri);
        var attributeMeta = attribute.meta;
        var firstDisplayFormMeta = attribute.content.displayForms[0].meta;
        return {
            attributeHeader: __assign(__assign({}, headerContent), { uri: firstDisplayFormMeta.uri, identifier: firstDisplayFormMeta.identifier, formOf: __assign(__assign({}, headerContent.formOf), { uri: attributeMeta.uri, identifier: attributeMeta.identifier }) })
        };
    };
    MockBuilder.prototype.mapMeasureGroupHeader = function (header) {
        var _this = this;
        var items = header.measureGroupHeader.items;
        return {
            measureGroupHeader: {
                items: items.map(function (measureHeaderItem) {
                    var headerItemContent = measureHeaderItem.measureHeaderItem;
                    var metricMeta = _this.schema.getMeasureMetaFor(headerItemContent.uri);
                    return {
                        measureHeaderItem: __assign(__assign({}, headerItemContent), { uri: metricMeta.uri, identifier: metricMeta.identifier })
                    };
                })
            }
        };
    };
    MockBuilder.prototype.replaceExecutionResultProjectId = function (resultLink) {
        var match = resultLink.match(/\/gdc\/app\/projects\/(.*)\/executionResults\//);
        var oldProjectId = match !== null && match.length > 0 ? match[match.length - 1] : null;
        return resultLink.replace(oldProjectId, this.schema.getProjectId());
    };
    MockBuilder.prototype.mapExecutionResult = function (wrappedExecutionResult) {
        var _this = this;
        var executionResult = wrappedExecutionResult.executionResult, headerItems = wrappedExecutionResult.executionResult.headerItems;
        return {
            executionResult: __assign(__assign({}, executionResult), { headerItems: headerItems.map(function (dimension) {
                    return _this.mapResultDimension(dimension);
                }) })
        };
    };
    MockBuilder.prototype.mapResultDimension = function (dimension) {
        var _this = this;
        return dimension.map(function (headers) { return _this.mapResultDimensionHeaders(headers); });
    };
    MockBuilder.prototype.mapResultDimensionHeaders = function (headers) {
        var _this = this;
        return headers.map(function (header) { return _this.mapResultHeader(header); });
    };
    MockBuilder.prototype.mapResultHeader = function (header) {
        if (typings_1.Execution.isAttributeHeaderItem(header)) {
            var attributeHeaderItem = header.attributeHeaderItem;
            return __assign(__assign({}, header), { attributeHeaderItem: __assign(__assign({}, attributeHeaderItem), { uri: this.schema.getAttributeElementUri(attributeHeaderItem.uri) }) });
        }
        return header;
    };
    return MockBuilder;
}());
function prettyPrint(jsValue) {
    return JSON.stringify(jsValue, null, 2);
}
function prettyPrintAsMock(jsObject) {
    return prettyPrint(jsObject)
        .replace(/"(.*)":/g, "$1:") // remove quotes from JSON keys
        .replace(/"/g, "'"); // replace string double quotes with apostrophes)
}
exports.mockBuilder = {
    register: function (app, project) {
        app.get("/mock-builder", function (_, response) {
            response
                .status(http_status_codes_1.OK)
                .set("Content-Type", "text/html")
                .send(mockBuilderUi_1["default"]);
        });
        app.post("/mock-builder/:projectId/:groupIndex", function (request, response) {
            if (request.params.projectId === project.project.meta.identifier) {
                var payload = request.body;
                var mockBuilder_1 = new MockBuilder(project, request.params
                    .groupIndex);
                var executionMock = mockBuilder_1.buildExecutionMockWithReplacedUris(payload);
                var stringMock = prettyPrintAsMock(executionMock);
                response
                    .status(http_status_codes_1.OK)
                    .set("Content-Type", "text/plain")
                    .send(stringMock);
            }
            else {
                response.sendStatus(http_status_codes_1.NOT_FOUND);
            }
        });
        return app;
    }
};
//# sourceMappingURL=mockBuilder.js.map