"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.permissions = exports.bootstrap = exports.DEFAULT_SEPARATORS = exports.getBootstrap = void 0;
var HttpStatusCodes = require("http-status-codes");
var profile_1 = require("../bear/profile");
var routes_1 = require("../../route/routes");
function getBootstrap(project) {
    var user = project.users[0];
    var accountSetting = profile_1.createAccountSettings(user);
    var featureFlags = project.featureFlags;
    var profileUri = accountSetting.accountSetting.links.self;
    var separators = user.separators;
    var projectId = project.project.meta.identifier;
    return {
        bootstrapResource: __assign(__assign({}, accountSetting), { hostnameBase: "secure.gooddata.com", settings: {
                applicationTitle: "GoodData",
                faviconUrl: "/images/favicon.ico",
                organizationName: "GoodData",
                displayFlashNews: true,
                logoUrl: "/images/header/logo.png",
                displayProjects: true,
                displayAccountPage: true,
                isBranded: false,
                supportEmail: "support@gooddata.com",
                supportForumUrl: "https://support.gooddata.com",
                privacyPolicyUrl: "https://www.gooddata.com/privacy.html",
                documentationUrl: "https://help.gooddata.com/doc/",
                securityStatementUrl: "https://www.gooddata.com/security.html",
                termsOfUseUrl: "https://www.gooddata.com/terms.html",
                trustUrl: "https://www.gooddata.com/trust/",
                appleTouchIconUrl: "/images/appleTouchIcon.png",
                skipClientRedirect: false,
                hideRegistration: false,
                useOnboarding: true,
                displayNPS: true
            }, profileSetting: {
                hints: {},
                projectSettings: {},
                currentProjectUri: routes_1.projectUri(projectId),
                npsLastParticipation: "2014-09-09 13:25:33",
                releaseNotice: [],
                links: {
                    self: profileUri + "/settings",
                    profile: profileUri
                },
                separators: separators
            }, current: {
                project: {
                    content: {
                        guidedNavigation: "1",
                        isPublic: "0",
                        state: "ENABLED",
                        driver: "Pg",
                        authorizationToken: "INTNA000000SRVS",
                        cluster: "",
                        environment: "PRODUCTION"
                    },
                    links: {
                        invitations: "/gdc/projects/" + projectId + "/invitations",
                        ldm: "/gdc/projects/" + projectId + "/ldm",
                        ldm_thumbnail: "/gdc/projects/" + projectId + "/ldm?thumbnail=1",
                        metadata: "/gdc/md/" + projectId,
                        roles: "/gdc/projects/" + projectId + "/roles",
                        self: "/gdc/projects/" + projectId,
                        userPermissions: "/gdc/projects/" + projectId + "/users/" + user.identifier + "/permissions",
                        userRoles: "/gdc/projects/" + projectId + "/users/" + user.identifier + "/roles",
                        users: "/gdc/projects/" + projectId + "/users?link=1&offset=0&limit=100",
                        templates: "/gdc/md/" + projectId + "/templates",
                        connectors: "/gdc/projects/" + projectId + "/connectors",
                        publicartifacts: "/gdc/projects/" + projectId + "/publicartifacts",
                        uploads: "https://secure-di.gooddata.com/project-uploads/" + projectId + "/",
                        dataload: "/gdc/projects/" + projectId + "/dataload",
                        schedules: "/gdc/projects/" + projectId + "/schedules",
                        eventstores: "/gdc/projects/" + projectId + "/dataload/eventstore/stores",
                        clearCaches: "/gdc/projects/" + projectId + "/clearCaches",
                        execute: "/gdc/projects/" + projectId + "/execute",
                        projectFeatureFlags: "/gdc/projects/" + projectId + "/projectFeatureFlags"
                    },
                    meta: {
                        created: "2014-07-14 14:45:24",
                        updated: "2015-03-14 11:12:00",
                        author: profileUri,
                        contributor: "/gdc/account/profile/876ec68f5630b38de65852ed5d6236ff",
                        summary: "",
                        title: project.project.meta.title
                    }
                },
                featureFlags: featureFlags,
                projectPermissions: {
                    permissions: project.projectPermissions,
                    links: {
                        project: routes_1.projectUri(projectId),
                        user: profileUri
                    }
                },
                clusterStatus: "ONLINE",
                timezone: {
                    id: "America/Los_Angeles",
                    displayName: "Pacific Standard Time",
                    shortDisplayName: "PST",
                    currentOffsetMs: -28800000
                },
                projectTemplates: [],
                dataUploadsInfo: {
                    statusesCount: {
                        OK: 15
                    }
                },
                loginMD5: user.identifier,
                integrations: [],
                projectIcons: [],
                requiresRedirect: false,
                analyticalDashboards: [],
                mapboxToken: process.env.MAPBOX_TOKEN || "dummy_mapbox_token"
            } })
    };
}
exports.getBootstrap = getBootstrap;
exports.DEFAULT_SEPARATORS = {
    thousand: ",",
    decimal: "."
};
exports.bootstrap = {
    register: function (app, project) {
        app.get("/gdc/app/account/bootstrap", function (req, res, next) {
            if (!req.query.projectUri ||
                req.query.projectUri === routes_1.projectUri(project.project.meta.identifier)) {
                var bootstrap_1 = getBootstrap(project);
                res.status(HttpStatusCodes.OK).json(bootstrap_1);
            }
            else {
                next();
            }
        });
        return app;
    }
};
exports.permissions = {
    register: function (app, project) {
        var projectId = project.project.meta.identifier;
        var user = project.users[0];
        app.get("/gdc/projects/" + projectId + "/users/:user/permissions", function (_req, res) {
            res.status(HttpStatusCodes.OK).json({
                associatedPermissions: {
                    links: {
                        user: "/gdc/account/profile/" + user.identifier,
                        project: "/gdc/projects/" + projectId + "/"
                    },
                    permissions: project.projectPermissions
                }
            });
        });
        return app;
    }
};
//# sourceMappingURL=bootstrap.js.map