"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.getDisplayFormByQualifier = exports.getDisplayForm = exports.getMetric = exports.isDerivedFromArithmeticMeasure = void 0;
// (C) 2007-2019 GoodData Corporation
var typings_1 = require("@gooddata/typings");
var invariant = require("invariant");
var MockProject_1 = require("../../../model/MockProject");
var Metric_1 = require("../../../model/Metric");
var AttributeDisplayForm_1 = require("../../../model/AttributeDisplayForm");
function findMeasure(afm, localIdentifier) {
    return afm.measures.find(function (measure) { return measure.localIdentifier === localIdentifier; });
}
function findMasterMeasureLocalIdentifierByDerivedMeasure(measure) {
    var definition = measure.definition;
    if (typings_1.ExecuteAFM.isPopMeasureDefinition(definition)) {
        return definition.popMeasure.measureIdentifier;
    }
    else if (typings_1.ExecuteAFM.isPreviousPeriodMeasureDefinition(definition)) {
        return definition.previousPeriodMeasure.measureIdentifier;
    }
    else {
        throw Error("This measure definition type is not supported! " + JSON.stringify(definition));
    }
}
function isDerivedFromArithmeticMeasure(measure, afm) {
    var measureDefinition = measure.definition;
    if (typings_1.ExecuteAFM.isPopMeasureDefinition(measureDefinition) ||
        typings_1.ExecuteAFM.isPreviousPeriodMeasureDefinition(measureDefinition)) {
        var masterMeasure = findMeasure(afm, findMasterMeasureLocalIdentifierByDerivedMeasure(measure));
        return typings_1.ExecuteAFM.isArithmeticMeasureDefinition(masterMeasure.definition);
    }
    return false;
}
exports.isDerivedFromArithmeticMeasure = isDerivedFromArithmeticMeasure;
function getMetric(project, measure, afm) {
    if (afm === void 0) { afm = null; }
    var measures = MockProject_1.getMetrics(project);
    var definition = measure.definition;
    if (typings_1.ExecuteAFM.isSimpleMeasureDefinition(definition)) {
        return getMetricBySimpleMeasure(definition, measures);
    }
    else {
        invariant(afm, "Could not read afm from execution");
        var referencedMeasure = findMeasure(afm, findMasterMeasureLocalIdentifierByDerivedMeasure(measure));
        invariant(referencedMeasure, "The measure could not be referenced.");
        return getMetricBySimpleMeasure(referencedMeasure.definition, measures);
    }
}
exports.getMetric = getMetric;
function getMetricBySimpleMeasure(measureDefinition, measures) {
    if (typings_1.ExecuteAFM.isSimpleMeasureDefinition(measureDefinition)) {
        if (typings_1.ExecuteAFM.isObjectUriQualifier(measureDefinition.measure.item)) {
            return Metric_1.getMetricByUri(measures, measureDefinition.measure.item.uri);
        }
        return Metric_1.getMetricByIdentifier(measures, measureDefinition.measure.item.identifier);
    }
    invariant(false, "Provided measure is not SimpleMeasureDefinition");
}
function getDisplayForm(project, attribute) {
    return getDisplayFormByQualifier(project, attribute.displayForm);
}
exports.getDisplayForm = getDisplayForm;
function getDisplayFormByQualifier(project, qualifier) {
    var attributeDisplayForms = MockProject_1.getAttributeDisplayForms(project);
    var dateDataSetDisplayForms = MockProject_1.getAllDateDataSetDisplayForms(project);
    var displayForms = __spreadArrays(attributeDisplayForms, dateDataSetDisplayForms);
    if (typings_1.ExecuteAFM.isObjectUriQualifier(qualifier)) {
        return AttributeDisplayForm_1.getDisplayFormByUri(displayForms, qualifier.uri);
    }
    return AttributeDisplayForm_1.getDisplayFormByIdentifier(displayForms, qualifier.identifier);
}
exports.getDisplayFormByQualifier = getDisplayFormByQualifier;
//# sourceMappingURL=afm.js.map