"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.generateDimensions = void 0;
var invariant = require("invariant");
var afm_1 = require("./afm");
var MockProject_1 = require("../../../model/MockProject");
var headers_1 = require("./helpers/headers");
function createMeasureHeader(project, afm) {
    var titles = headers_1.getMeasureTitles(project, afm);
    return {
        measureGroupHeader: {
            items: afm.measures.map(function (measure) {
                // for ad hoc measures backend doesn't return uri and identifier
                var isAdHoc = headers_1.isAdHocMeasure(measure);
                if (headers_1.isAttributeBasedMeasure(measure) ||
                    headers_1.isArithmeticMeasure(measure) ||
                    afm_1.isDerivedFromArithmeticMeasure(measure, afm)) {
                    return {
                        measureHeaderItem: {
                            localIdentifier: measure.localIdentifier,
                            format: measure.format,
                            name: titles.get(measure.localIdentifier)
                        }
                    };
                }
                var metric = afm_1.getMetric(project, measure, afm);
                return {
                    measureHeaderItem: {
                        identifier: isAdHoc ? undefined : metric.meta.identifier,
                        uri: isAdHoc ? undefined : metric.meta.uri,
                        localIdentifier: measure.localIdentifier,
                        format: measure.format || metric.content.format,
                        name: titles.get(measure.localIdentifier)
                    }
                };
            })
        }
    };
}
function createAttributeHeader(project, afm, itemIdentifier, totalTypes) {
    if (totalTypes === void 0) { totalTypes = []; }
    var attribute = afm.attributes.find(function (attribute) { return attribute.localIdentifier === itemIdentifier; });
    invariant(attribute, "Attribute '" + itemIdentifier + "' was not found in AFM");
    var displayForm = afm_1.getDisplayForm(project, attribute);
    invariant(displayForm, "Display form '" + attribute.alias + "' not found");
    var allAttributes = __spreadArrays(MockProject_1.getAttributes(project), MockProject_1.getDateAttributes(project));
    var attributeInfo = allAttributes.find(function (attr) { return attr.meta.uri === displayForm.content.formOf; });
    invariant(attributeInfo, "Attribute info '" + displayForm.content.formOf + "' not found");
    var totalItems = totalTypes.map(function (name) { return ({ totalHeaderItem: { name: name } }); });
    return {
        attributeHeader: {
            identifier: displayForm.meta.identifier,
            uri: displayForm.meta.uri,
            name: displayForm.meta.title,
            localIdentifier: attribute.localIdentifier,
            totalItems: totalItems.length ? totalItems : undefined,
            formOf: {
                uri: attributeInfo.meta.uri,
                identifier: attributeInfo.meta.identifier,
                name: attribute.alias || attributeInfo.meta.title
            }
        }
    };
}
function createHeader(project, afm, itemIdentifier, totalTypes) {
    if (totalTypes === void 0) { totalTypes = []; }
    if (itemIdentifier === "measureGroup") {
        return createMeasureHeader(project, afm);
    }
    return createAttributeHeader(project, afm, itemIdentifier, totalTypes);
}
function generateDimensions(project, afm, resultSpec, totalTypes) {
    if (totalTypes === void 0) { totalTypes = []; }
    return resultSpec.dimensions.map(function (dimension) {
        return {
            headers: dimension.itemIdentifiers.map(function (itemIdentifier) {
                return createHeader(project, afm, itemIdentifier, totalTypes);
            })
        };
    });
}
exports.generateDimensions = generateDimensions;
//# sourceMappingURL=dimensions.js.map