"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.relativizeMockedExecutionLocalIdentifiers = exports.isRelativelyEqual = exports.isILocalIdentifierQualifier = void 0;
// (C) 2007-2020 GoodData Corporation
var typings_1 = require("@gooddata/typings");
var lodash_1 = require("lodash");
var SchemaAfmExecution_1 = require("../../../schema/model/SchemaAfmExecution");
var executeAfm_1 = require("../executeAfm");
var RELATIVIZED_ALIAS = "relativized-alias";
function relativizeTotals(totals, replacedKeys) {
    if (totals === undefined) {
        return {};
    }
    return {
        totals: totals.map(function (totalItem) {
            var measureIdentifier = totalItem.measureIdentifier, attributeIdentifier = totalItem.attributeIdentifier;
            return __assign(__assign({}, totalItem), { measureIdentifier: replacedKeys.get(measureIdentifier), attributeIdentifier: replacedKeys.get(attributeIdentifier) });
        })
    };
}
function relativizeAlias(enabled) {
    if (!enabled) {
        return {};
    }
    return {
        alias: RELATIVIZED_ALIAS
    };
}
function isILocalIdentifierQualifier(objQualifier) {
    return (!lodash_1.isEmpty(objQualifier) &&
        objQualifier.localIdentifier !== undefined);
}
exports.isILocalIdentifierQualifier = isILocalIdentifierQualifier;
function isFilterReferencingLocalAfmMeasure(filter) {
    return ((typings_1.ExecuteAFM.isMeasureValueFilter(filter) &&
        isILocalIdentifierQualifier(filter.measureValueFilter.measure)) ||
        (typings_1.ExecuteAFM.isRankingFilter(filter) && filter.rankingFilter.measures.some(isILocalIdentifierQualifier)));
}
function relativizeIdentifiers(execution, replacedKeys, relativizeAliases, stripLocalMeasureValueAndRankingFilters) {
    if (stripLocalMeasureValueAndRankingFilters === void 0) { stripLocalMeasureValueAndRankingFilters = false; }
    var cloned = executeAfm_1.sanitize(lodash_1.cloneDeep(execution));
    if (cloned.execution.afm.measures) {
        cloned.execution.afm.measures = cloned.execution.afm.measures.map(function (measure, index) {
            var originalKey = measure.localIdentifier;
            var newKey = "m" + (index + 1); // index measures from 1 (m1, m2, m3) to match our unwritten conventions
            replacedKeys.set(originalKey, newKey);
            return __assign(__assign(__assign({}, measure), { localIdentifier: newKey }), relativizeAlias(relativizeAliases));
        });
        cloned.execution.afm.measures.forEach(function (measure) {
            if (typings_1.ExecuteAFM.isPreviousPeriodMeasureDefinition(measure.definition)) {
                var definition = measure.definition;
                var ref = definition.previousPeriodMeasure.measureIdentifier;
                definition.previousPeriodMeasure.measureIdentifier = replacedKeys.get(ref);
            }
            else if (typings_1.ExecuteAFM.isPopMeasureDefinition(measure.definition)) {
                var definition = measure.definition;
                var ref = definition.popMeasure.measureIdentifier;
                definition.popMeasure.measureIdentifier = replacedKeys.get(ref);
            }
            else if (typings_1.ExecuteAFM.isArithmeticMeasureDefinition(measure.definition)) {
                var definition = measure.definition;
                var refs = definition.arithmeticMeasure.measureIdentifiers;
                definition.arithmeticMeasure.measureIdentifiers = refs.map(function (ref) { return replacedKeys.get(ref); });
            }
        });
    }
    if (cloned.execution.afm.attributes) {
        cloned.execution.afm.attributes = cloned.execution.afm.attributes.map(function (attribute, index) {
            var originalKey = attribute.localIdentifier;
            var newKey = "a" + (index + 1); // index attributes from 1 (a1, a2, a3) to match our unwritten conventions
            replacedKeys.set(originalKey, newKey);
            return __assign(__assign(__assign({}, attribute), { localIdentifier: newKey }), relativizeAlias(relativizeAliases));
        });
    }
    if (stripLocalMeasureValueAndRankingFilters && cloned.execution.afm.filters) {
        cloned.execution.afm.filters = cloned.execution.afm.filters.filter(function (filter) { return !isFilterReferencingLocalAfmMeasure(filter); });
    }
    if (!cloned.execution.resultSpec) {
        return cloned;
    }
    if (cloned.execution.resultSpec.dimensions) {
        cloned.execution.resultSpec.dimensions = cloned.execution.resultSpec.dimensions.map(function (dimension) {
            return __assign(__assign(__assign({}, dimension), { itemIdentifiers: dimension.itemIdentifiers.map(function (itemIdentifier) {
                    if (replacedKeys.has(itemIdentifier)) {
                        return replacedKeys.get(itemIdentifier);
                    }
                    return itemIdentifier;
                }) }), relativizeTotals(dimension.totals, replacedKeys));
        });
    }
    if (cloned.execution.resultSpec.sorts) {
        cloned.execution.resultSpec.sorts = cloned.execution.resultSpec.sorts.map(function (sort) {
            if (typings_1.ExecuteAFM.isAttributeSortItem(sort)) {
                return {
                    attributeSortItem: {
                        direction: sort.attributeSortItem.direction,
                        attributeIdentifier: replacedKeys.get(sort.attributeSortItem.attributeIdentifier)
                    }
                };
            }
            if (typings_1.ExecuteAFM.isMeasureSortItem(sort)) {
                return {
                    measureSortItem: {
                        direction: sort.measureSortItem.direction,
                        locators: sort.measureSortItem.locators.map(function (locatorItem) {
                            if (typings_1.ExecuteAFM.isMeasureLocatorItem(locatorItem)) {
                                return {
                                    measureLocatorItem: {
                                        measureIdentifier: replacedKeys.get(locatorItem.measureLocatorItem.measureIdentifier)
                                    }
                                };
                            }
                            return {
                                attributeLocatorItem: __assign(__assign({}, locatorItem.attributeLocatorItem), { attributeIdentifier: locatorItem.attributeLocatorItem.attributeIdentifier })
                            };
                        })
                    }
                };
            }
        });
    }
    return cloned;
}
function relativizeResponseIdentifiers(executionResponse, replacedKeys) {
    if (SchemaAfmExecution_1.isExecutionResponseWrapper(executionResponse)) {
        var cloned = lodash_1.cloneDeep(executionResponse);
        cloned.executionResponse.dimensions.forEach(function (dimension) {
            dimension.headers.forEach(function (header) {
                if (typings_1.Execution.isMeasureGroupHeader(header)) {
                    var items = header.measureGroupHeader.items;
                    items.forEach(function (measureItem) {
                        var measureHeaderItem = measureItem.measureHeaderItem;
                        measureHeaderItem.localIdentifier = replacedKeys.get(measureHeaderItem.localIdentifier);
                    });
                }
                if (typings_1.Execution.isAttributeHeader(header)) {
                    header.attributeHeader.localIdentifier = replacedKeys.get(header.attributeHeader.localIdentifier);
                }
            });
        });
        return cloned;
    }
    return executionResponse;
}
/**
 * Compares two execution objects ignoring differences in identifiers
 *
 * This is necessary due to the fact that our client apps (namely AD) creates
 * unique local identifiers every single session.
 */
function isRelativelyEqual(execution, other) {
    return lodash_1.isEqual(relativizeIdentifiers(execution, new Map(), true, true), relativizeIdentifiers(other, new Map(), true, true));
}
exports.isRelativelyEqual = isRelativelyEqual;
function relativizeMockedExecutionLocalIdentifiers(mockedExecution) {
    var execution = mockedExecution.execution, executionResponse = mockedExecution.executionResponse, executionResult = mockedExecution.executionResult;
    var replacedKeys = new Map();
    return __assign(__assign({}, mockedExecution), { execution: relativizeIdentifiers(execution, replacedKeys, false), executionResponse: relativizeResponseIdentifiers(executionResponse, replacedKeys), executionResult: executionResult });
}
exports.relativizeMockedExecutionLocalIdentifiers = relativizeMockedExecutionLocalIdentifiers;
//# sourceMappingURL=execution.js.map