"use strict";
exports.__esModule = true;
exports.getMeasureTitles = exports.isAdHocMeasure = exports.isArithmeticMeasure = exports.isAttributeBasedMeasure = void 0;
// (C) 2007-2019 GoodData Corporation
var invariant = require("invariant");
var lodash_1 = require("lodash");
var typings_1 = require("@gooddata/typings");
var afm_1 = require("../afm");
var map_1 = require("../../../../utils/map");
function isAttributeBasedMeasure(measure) {
    if (typings_1.ExecuteAFM.isSimpleMeasureDefinition(measure.definition)) {
        return measure.definition.measure.aggregation === "count";
    }
    return false;
}
exports.isAttributeBasedMeasure = isAttributeBasedMeasure;
function isArithmeticMeasure(measure) {
    return typings_1.ExecuteAFM.isArithmeticMeasureDefinition(measure.definition);
}
exports.isArithmeticMeasure = isArithmeticMeasure;
function isAdHocMeasure(measure) {
    var measureDefinition = measure.definition;
    return (typings_1.ExecuteAFM.isPopMeasureDefinition(measureDefinition) ||
        typings_1.ExecuteAFM.isPreviousPeriodMeasureDefinition(measureDefinition) ||
        typings_1.ExecuteAFM.isArithmeticMeasureDefinition(measureDefinition) ||
        !lodash_1.isEmpty(measureDefinition.measure.filters) ||
        !!measureDefinition.measure.aggregation ||
        !!measureDefinition.measure.computeRatio);
}
exports.isAdHocMeasure = isAdHocMeasure;
function buildTitlesMap(project, afm) {
    var titles = new Map();
    for (var _i = 0, _a = afm.measures; _i < _a.length; _i++) {
        var measure = _a[_i];
        // Attribute based measures, arithmetic measures and derived measures from arithmetic measure
        // have title defined in AFM `measure.alias`
        if (isAttributeBasedMeasure(measure) ||
            isArithmeticMeasure(measure) ||
            afm_1.isDerivedFromArithmeticMeasure(measure, afm)) {
            continue;
        }
        var metric = afm_1.getMetric(project, measure, afm);
        invariant(metric, "Metric '" + measure.localIdentifier + "' not found");
        titles.set(measure.localIdentifier, metric.meta.title);
    }
    return titles;
}
function buildAliasesMap(afm) {
    var aliases = new Map();
    for (var _i = 0, _a = afm.measures; _i < _a.length; _i++) {
        var measure = _a[_i];
        if (measure.alias) {
            aliases.set(measure.localIdentifier, measure.alias);
        }
    }
    return aliases;
}
function getMeasureTitles(project, afm) {
    return map_1.merge(buildTitlesMap(project, afm), buildAliasesMap(afm));
}
exports.getMeasureTitles = getMeasureTitles;
//# sourceMappingURL=headers.js.map