"use strict";
exports.__esModule = true;
exports.getTotalsData = exports.getTotals = exports.getTotalsTypesFromExecution = exports.getTotalsFromExecution = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var typings_1 = require("@gooddata/typings");
var HttpStatusCodes = require("http-status-codes");
function getTotalsFromExecution(normalizedExecution) {
    return lodash_1.get(normalizedExecution, "execution.resultSpec.dimensions[0].totals", []);
}
exports.getTotalsFromExecution = getTotalsFromExecution;
function getTotalsTypesFromExecution(normalizedExecution) {
    var totals = getTotalsFromExecution(normalizedExecution);
    var totalsTypes = totals.map(function (total) { return total.type; });
    return lodash_1.uniq(totalsTypes);
}
exports.getTotalsTypesFromExecution = getTotalsTypesFromExecution;
function getTotals(normalizedExecution, dataResult, randomGenerator) {
    var types = getTotalsTypesFromExecution(normalizedExecution);
    var data = getTotalsData(normalizedExecution, dataResult, randomGenerator);
    // just basic check for afm.nativeTotals field, not checking if afm.nativeTotals match resultSpec.totals definition
    if (lodash_1.includes(types, "nat") && lodash_1.isEmpty(normalizedExecution.execution.afm.nativeTotals)) {
        return {
            error: {
                statusCode: HttpStatusCodes.BAD_REQUEST,
                message: "Missing AFM.nativeTotal field"
            }
        };
    }
    return { types: types, data: data };
}
exports.getTotals = getTotals;
function getTotalsData(execution, dataResult, randomGenerator) {
    var _a = execution.execution, resultSpec = _a.resultSpec, afm = _a.afm;
    var dimensionsFlattened = lodash_1.flattenDepth(dataResult.headerItems, 2);
    var measureHeaderItems = dimensionsFlattened.filter(function (headerItem) {
        return typings_1.Execution.isMeasureHeaderItem(headerItem);
    });
    var totalTypes = ["sum", "max", "min", "avg", "med", "nat"];
    return dataResult.headerItems.map(function (_, headerDimensionIndex) {
        if (resultSpec.dimensions.length === 0) {
            return [];
        }
        var totals = resultSpec.dimensions[headerDimensionIndex]
            ? resultSpec.dimensions[headerDimensionIndex].totals
            : [];
        if (totals && totals.length > 0) {
            return totalTypes
                .filter(function (totalType) { return totals.some(function (total) { return total.type === totalType; }); })
                .map(function (totalType) {
                return measureHeaderItems.map(function (measureHeaderItem) {
                    var localIdentifier = afm.measures && afm.measures[measureHeaderItem.measureHeaderItem.order]
                        ? afm.measures[measureHeaderItem.measureHeaderItem.order].localIdentifier
                        : null;
                    if (!localIdentifier) {
                        return null;
                    }
                    var isTotalDefined = totals.some(function (total) { return total.measureIdentifier === localIdentifier && total.type === totalType; });
                    return isTotalDefined ? randomGenerator({ totalType: totalType, localIdentifier: localIdentifier }) : null;
                });
            });
        }
        return [];
    });
}
exports.getTotalsData = getTotalsData;
//# sourceMappingURL=totals.js.map