"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
// (C) 2019 GoodData Corporation
var lodash_1 = require("lodash");
var typings_1 = require("@gooddata/typings");
var FilterConverter_1 = require("./FilterConverter");
var MeasureConverter = {
    convertMeasure: convertMeasure,
    getFormat: getFormat
};
exports["default"] = MeasureConverter;
function convertMeasure(measure) {
    var definition = measure.measure.definition;
    var convertedDefinition = convertMeasureDefinition(definition);
    var format = getFormat(measure);
    var formatProp = format ? { format: format } : {};
    var alias = measure.measure.alias ? measure.measure.alias : measure.measure.title;
    var aliasProp = alias ? { alias: alias } : {};
    return __assign(__assign({ localIdentifier: measure.measure.localIdentifier, definition: convertedDefinition }, aliasProp), formatProp);
}
function convertMeasureDefinition(definition) {
    if (typings_1.VisualizationObject.isMeasureDefinition(definition)) {
        return convertSimpleMeasureDefinition(definition);
    }
    else if (typings_1.VisualizationObject.isPopMeasureDefinition(definition)) {
        return convertPopMeasureDefinition(definition);
    }
    else if (typings_1.VisualizationObject.isPreviousPeriodMeasureDefinition(definition)) {
        return convertPreviousPeriodMeasureDefinition(definition);
    }
    else if (typings_1.VisualizationObject.isArithmeticMeasureDefinition(definition)) {
        return convertArithmeticMeasureDefinition(definition);
    }
    else {
        throw Error("The measure definition is not supported: " + JSON.stringify(definition));
    }
}
function convertSimpleMeasureDefinition(definition) {
    var measureDefinition = definition.measureDefinition;
    var filters = measureDefinition.filters
        ? lodash_1.compact(measureDefinition.filters.map(FilterConverter_1.convertVisualizationObjectFilter))
        : [];
    var filtersProp = filters.length ? { filters: filters } : {};
    var aggregation = measureDefinition.aggregation;
    var aggregationProp = aggregation ? { aggregation: aggregation } : {};
    var computeRatio = measureDefinition.computeRatio;
    var computeRatioProp = computeRatio ? { computeRatio: computeRatio } : {};
    return {
        measure: __assign(__assign(__assign({ item: measureDefinition.item }, filtersProp), aggregationProp), computeRatioProp)
    };
}
function convertPopMeasureDefinition(definition) {
    var popMeasureDefinition = definition.popMeasureDefinition;
    return {
        popMeasure: {
            measureIdentifier: popMeasureDefinition.measureIdentifier,
            popAttribute: popMeasureDefinition.popAttribute
        }
    };
}
function convertPreviousPeriodMeasureDefinition(definition) {
    var previousPeriodMeasure = definition.previousPeriodMeasure;
    return {
        previousPeriodMeasure: {
            measureIdentifier: previousPeriodMeasure.measureIdentifier,
            dateDataSets: previousPeriodMeasure.dateDataSets.map(function (dateDataSet) { return ({
                dataSet: dateDataSet.dataSet,
                periodsAgo: dateDataSet.periodsAgo
            }); })
        }
    };
}
function convertArithmeticMeasureDefinition(definition) {
    var arithmeticMeasure = definition.arithmeticMeasure;
    return {
        arithmeticMeasure: {
            measureIdentifiers: arithmeticMeasure.measureIdentifiers.slice(),
            operator: arithmeticMeasure.operator
        }
    };
}
function getFormat(measure) {
    var definition = measure.measure.definition;
    var measureFormat = lodash_1.get(measure.measure, "format");
    if (typings_1.VisualizationObject.isArithmeticMeasureDefinition(definition)) {
        if (definition.arithmeticMeasure.operator === "change") {
            return "#,##0.00%";
        }
    }
    var predefinedFormat = typings_1.VisualizationObject.isMeasureDefinition(definition)
        ? getPredefinedFormat(definition)
        : undefined;
    return predefinedFormat || measureFormat;
}
function getPredefinedFormat(definition) {
    var measureDefinition = definition.measureDefinition;
    // should we prefer format defined on measure? If so, fix computeRatio format in AD
    return measureDefinition.computeRatio
        ? "#,##0.00%"
        : measureDefinition.aggregation === "count"
            ? "#,##0"
            : null;
}
//# sourceMappingURL=MeasureConverter.js.map