"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.mergeFiltersToAfm = exports.appendFilters = exports.dateFiltersDataSetsMatch = exports.getDateFilterDateDataSet = exports.getId = exports.ALL_TIME_GRANULARITY = void 0;
// (C) 2019-2020 GoodData Corporation
var typings_1 = require("@gooddata/typings");
var lodash_1 = require("lodash");
exports.ALL_TIME_GRANULARITY = "ALL_TIME_GRANULARITY";
function getDateFilter(filters) {
    if (filters === void 0) { filters = []; }
    return filters.find(typings_1.ExecuteAFM.isDateFilter);
}
function getAttributeFilters(filters) {
    if (filters === void 0) { filters = []; }
    return filters.filter(typings_1.ExecuteAFM.isAttributeFilter);
}
var getMeasureValueFilters = function (filters) {
    if (filters === void 0) { filters = []; }
    return filters.filter(typings_1.ExecuteAFM.isMeasureValueFilter);
};
var getRankingFilters = function (filters) {
    if (filters === void 0) { filters = []; }
    return filters.filter(typings_1.ExecuteAFM.isRankingFilter);
};
/**
 * Return uri or identifier from ObjQualifier
 *
 * @method getId
 * @param {AFM.ObjQualifier} obj
 * @returns {string|null}
 */
function getId(obj) {
    if (obj.uri) {
        return obj.uri;
    }
    if (obj.identifier) {
        return obj.identifier;
    }
    return null;
}
exports.getId = getId;
/**
 * Returns date filter date dataset
 *
 * @method getDateFilterDateDataSet
 * @param {AFM.DateFilterItem} filter
 * @returns {AFM.ObjQualifier | null }
 */
function getDateFilterDateDataSet(filter) {
    if (typings_1.ExecuteAFM.isRelativeDateFilter(filter)) {
        return filter.relativeDateFilter.dataSet;
    }
    if (typings_1.ExecuteAFM.isAbsoluteDateFilter(filter)) {
        return filter.absoluteDateFilter.dataSet;
    }
    throw new Error("Unsupported type of date filter");
}
exports.getDateFilterDateDataSet = getDateFilterDateDataSet;
/**
 * Returns true if dateFilters dataSets match
 *
 * @method dateFiltersDataSetsMatch
 * @param {AFM.DateFilterItem} f1
 * @param {AFM.DateFilterItem} f2
 * @returns {AFM.ObjQualifier | null | boolean}
 */
function dateFiltersDataSetsMatch(f1, f2) {
    var d1 = getDateFilterDateDataSet(f1);
    var d2 = getDateFilterDateDataSet(f2);
    return d1 && d2 && getId(d1) === getId(d2);
}
exports.dateFiltersDataSetsMatch = dateFiltersDataSetsMatch;
function isDateFilterAllTime(dateFilter) {
    if (typings_1.ExecuteAFM.isRelativeDateFilter(dateFilter)) {
        return dateFilter.relativeDateFilter.granularity === exports.ALL_TIME_GRANULARITY;
    }
    return false;
}
/**
 * Append attribute filters and date filter to afm
 *
 * Date filter handling:
 *      - Override if date filter has the same id
 *      - Add if date filter if date filter id is different
 *
 * Attribute filter handling:
 *      - Add all
 *
 * @method appendFilters
 * @param {AFM.IAfm} afm
 * @param {AFM.AttributeFilterItem[]} attributeFilters
 * @param {AFM.DateFilterItem} dateFilter
 * @param {AFM.IMeasureValueFilter} measureValueFilters
 * @param {AFM.IRankingFilter} rankingFilters
 * @return {AFM.IAfm}
 */
function appendFilters(afm, attributeFilters, dateFilter, measureValueFilters, rankingFilters) {
    var dateFilters = dateFilter && !isDateFilterAllTime(dateFilter) ? [dateFilter] : [];
    var afmDateFilter = afm.filters ? afm.filters.filter(typings_1.ExecuteAFM.isDateFilter)[0] : null;
    // all-time selected, need to delete date filter from filters
    var afmFilters = afm.filters || [];
    if (dateFilter && isDateFilterAllTime(dateFilter)) {
        afmFilters = afmFilters.filter(function (filter) {
            if (typings_1.ExecuteAFM.isDateFilter(filter)) {
                return !dateFiltersDataSetsMatch(filter, dateFilter);
            }
            return true;
        });
    }
    if ((afmDateFilter && dateFilter && !dateFiltersDataSetsMatch(afmDateFilter, dateFilter)) ||
        (afmDateFilter && !dateFilter)) {
        dateFilters.push(afmDateFilter);
    }
    var afmNonDateFilters = afmFilters.filter(function (filter) { return !typings_1.ExecuteAFM.isDateFilter(filter); });
    var filters = lodash_1.compact(__spreadArrays(afmNonDateFilters, attributeFilters, dateFilters, (measureValueFilters || []), (rankingFilters || [])));
    if (filters.length || (afm.filters && afm.filters.length)) {
        return __assign(__assign({}, afm), { filters: filters });
    }
    return afm;
}
exports.appendFilters = appendFilters;
function mergeFiltersToAfm(afm, additionalFilters) {
    if (additionalFilters === void 0) { additionalFilters = []; }
    var attributeFilters = getAttributeFilters(additionalFilters);
    var dateFilter = getDateFilter(additionalFilters);
    var measureValueFilters = getMeasureValueFilters(additionalFilters);
    var rankingFilters = getRankingFilters(additionalFilters);
    return appendFilters(afm, attributeFilters, dateFilter, measureValueFilters, rankingFilters);
}
exports.mergeFiltersToAfm = mergeFiltersToAfm;
//# sourceMappingURL=mergeFilters.js.map