"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.toAfm = void 0;
// (C) 2019 GoodData Corporation
var typings_1 = require("@gooddata/typings");
var lodash_1 = require("lodash");
var FilterConverter_1 = require("./FilterConverter");
var MeasureConverter_1 = require("./MeasureConverter");
function convertAttribute(attribute, idx) {
    var alias = attribute.visualizationAttribute.alias;
    var aliasProp = alias ? { alias: alias } : {};
    return __assign({ displayForm: attribute.visualizationAttribute.displayForm, localIdentifier: attribute.visualizationAttribute.localIdentifier || "a" + (idx + 1) }, aliasProp);
}
function getMeasures(buckets) {
    return buckets.reduce(function (result, bucket) {
        var measureItems = bucket.items.filter(typings_1.VisualizationObject.isMeasure);
        return result.concat(measureItems);
    }, []);
}
function getNativeTotalAttributeIdentifiers(bucket, total) {
    var attributes = bucket.items.filter(typings_1.VisualizationObject.isAttribute);
    var totalAttributeIndex = attributes.findIndex(function (attribute) { return attribute.visualizationAttribute.localIdentifier === total.attributeIdentifier; });
    return attributes
        .slice(0, totalAttributeIndex)
        .map(function (attribute) { return attribute.visualizationAttribute.localIdentifier; });
}
function convertNativeTotals(visObj) {
    var nativeTotalsPerBucket = visObj.buckets.map(function (bucket) {
        var totals = bucket.totals || [];
        var nativeTotals = totals.filter(function (total) { return total.type === "nat"; });
        return nativeTotals.map(function (total) { return ({
            measureIdentifier: total.measureIdentifier,
            attributeIdentifiers: getNativeTotalAttributeIdentifiers(bucket, total)
        }); });
    });
    return lodash_1.flatMap(nativeTotalsPerBucket);
}
function getAttributes(buckets) {
    return buckets.reduce(function (result, bucket) {
        var items = bucket.items.filter(typings_1.VisualizationObject.isAttribute);
        return result.concat(items);
    }, []);
}
function toAfm(visualizationObject) {
    var attributes = getAttributes(visualizationObject.buckets).map(convertAttribute);
    var attrProp = attributes.length ? { attributes: attributes } : {};
    var measures = getMeasures(visualizationObject.buckets).map(MeasureConverter_1["default"].convertMeasure);
    var measuresProp = measures.length ? { measures: measures } : {};
    var filters = visualizationObject.filters
        ? lodash_1.compact(visualizationObject.filters.map(FilterConverter_1.convertVisualizationObjectFilter))
        : [];
    var filtersProp = filters.length ? { filters: filters } : {};
    var nativeTotals = convertNativeTotals(visualizationObject);
    var nativeTotalsProp = nativeTotals.length ? { nativeTotals: nativeTotals } : {};
    return __assign(__assign(__assign(__assign({}, measuresProp), attrProp), filtersProp), nativeTotalsProp);
}
exports.toAfm = toAfm;
//# sourceMappingURL=toAfm.js.map