"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.executions = exports.findMatchingExecution = void 0;
var lodash_1 = require("lodash");
var HttpStatusCodes = require("http-status-codes");
var MockProject_1 = require("../../model/MockProject");
var routes_1 = require("../../route/routes");
// TODO: Do we still need it? :-/
var otfExecutionsCache = [];
var DEFAULT_NUMBER_OF_ROWS = 1;
var executionCache = new Map();
function getTitle(identifier, definitions) {
    if (definitions === void 0) { definitions = []; }
    var definition = definitions.find(function (definition) { return lodash_1.get(definition, "metricDefinition.identifier") === identifier; });
    return lodash_1.get(definition, "metricDefinition.title") || "Title " + identifier;
}
var getIdentifier = function (uri) { return lodash_1.last(uri.split("/")); };
function getHeaders(requestColumns, mockExecution, body) {
    var objectMapping = mockExecution.objectMapping;
    return requestColumns.map(function (identifier) { return ({
        type: objectMapping[identifier] === "metric" ? "metric" : "attrLabel",
        id: getIdentifier(identifier),
        title: getTitle(getIdentifier(identifier), lodash_1.get(body, "execution.definitions")),
        uri: identifier
    }); });
}
function getExecutionResponse(requestColumns, executionId, mockExecution, projectId, body) {
    return {
        executionResult: {
            headers: getHeaders(requestColumns, mockExecution, body),
            tabularDataResult: routes_1.tabularResultUri(projectId, executionId),
            extendedTabularDataResult: routes_1.extendedTabularResultUri(projectId, executionId)
        }
    };
}
function createExecutionOnTheFly(requestColumns, project) {
    var displayForms = MockProject_1.getAttributes(project).reduce(function (a, b) { return __spreadArrays(a, b.content.displayForms); }, []);
    var objectMapping = requestColumns.reduce(function (mapping, uriOrIdentifier) {
        var _a;
        var isColumnDf = displayForms.some(function (df) {
            return df.meta.identifier === uriOrIdentifier || df.meta.uri === uriOrIdentifier;
        });
        return __assign(__assign({}, mapping), (_a = {}, _a[uriOrIdentifier] = isColumnDf ? "attrLabel" : "metric", _a));
    }, {});
    return {
        result: "ok",
        objectMapping: objectMapping,
        columns: requestColumns
    };
}
function uriToId(uri) {
    return uri.substring(uri.lastIndexOf("/") + 1, uri.length);
}
function findMatchingExecution(requestedExecution, availableExecutions) {
    var requestedColumnIds = __spreadArrays(requestedExecution.columns).map(uriToId).sort();
    var requestedWhere = !lodash_1.isEmpty(requestedExecution.where)
        ? lodash_1.mapKeys(requestedExecution.where, function (_, key) { return uriToId(key); })
        : undefined;
    return availableExecutions.find(function (e) { return lodash_1.isEqual(requestedColumnIds, lodash_1.keys(e.objectMapping).sort()) && lodash_1.isEqual(requestedWhere, e.where); });
}
exports.findMatchingExecution = findMatchingExecution;
function createResponse(body, project, projectId, executionId) {
    var mockExecutions = __spreadArrays(project.executions, otfExecutionsCache);
    var requestColumns = body.execution.columns;
    var mockExecution = findMatchingExecution(body.execution, mockExecutions);
    if (!mockExecution) {
        // default execution
        mockExecution = createExecutionOnTheFly(requestColumns, project);
        otfExecutionsCache.push(mockExecution);
    }
    executionCache.set(executionId, {
        pollCounter: 0,
        execution: mockExecution
    });
    return getExecutionResponse(requestColumns, executionId, mockExecution, projectId, body);
}
var isUri = function (str) { return str.startsWith("/gdc"); };
function getRowsCount(project, execution) {
    var ensureIdentifier = function (str) { return (isUri(str) ? getIdentifier(str) : str); };
    return execution.columns
        .map(function (column) { return ensureIdentifier(column); })
        .filter(function (column) { return MockProject_1.isKnownDisplayFormId(project, column); })
        .map(function (column) { return MockProject_1.getAttributeElementsByDisplayForm(project, column).length; })
        .reduce(function (prev, next) { return Math.max(prev, next); }, DEFAULT_NUMBER_OF_ROWS);
}
function generateValues(project, execution, generateElementFn, orderBy) {
    var columnIdentifiers = execution.columns;
    var values = [];
    var rowsCount = getRowsCount(project, execution);
    // Create table and sort it if orderBy is defined
    var table = generateTable(project, execution, orderBy);
    // tslint:disable-next-line:no-param-reassign
    for (var row = 0; row < rowsCount; row += 1) {
        var columns = [];
        // tslint:disable-next-line:no-param-reassign
        for (var column = 0; column < columnIdentifiers.length; column += 1) {
            var columnType = execution.objectMapping[columnIdentifiers[column]];
            var value = table[row][columnIdentifiers[column]];
            if (columnType === "metric") {
                columns.push(value);
            }
            else {
                columns.push(generateElementFn(row, column, value));
            }
        }
        values.push(columns);
    }
    return values;
}
function generateTable(project, execution, orderBy) {
    var rowsCount = getRowsCount(project, execution);
    var columnIdentifiers = execution.columns;
    var table = [];
    // tslint:disable-next-line:no-param-reassign
    for (var row = 0; row < rowsCount; row += 1) {
        var rowValue = {};
        // tslint:disable-next-line:no-param-reassign
        for (var _i = 0, columnIdentifiers_1 = columnIdentifiers; _i < columnIdentifiers_1.length; _i++) {
            var columnIdentifier = columnIdentifiers_1[_i];
            var columnType = execution.objectMapping[columnIdentifier];
            var value = void 0;
            if (columnType === "metric") {
                value =
                    execution.resultValues && execution.resultValues[row]
                        ? execution.resultValues[row]
                        : (value = Math.random() * 100);
            }
            else {
                value = "Element (" + getIdentifier(columnIdentifier) + ") " + row;
            }
            rowValue[columnIdentifier] = value;
        }
        table.push(rowValue);
    }
    if (orderBy) {
        var sortedTable = lodash_1.sortBy(table, ["" + orderBy.column]);
        return orderBy.direction === "asc" ? sortedTable : sortedTable.reverse();
    }
    else {
        return table;
    }
}
function createResult(project, execution, orderBy) {
    var values = generateValues(project, execution, function (_row, _column, identifier) { return identifier; }, orderBy);
    return {
        tabularDataResult: {
            values: values
        }
    };
}
function createExtendedResult(projectId, project, execution, id, orderBy) {
    var values = generateValues(project, execution, function (row, column, identifier) { return ({
        id: 10 * column + row,
        name: identifier
    }); }, orderBy);
    return {
        extendedTabularDataResult: {
            links: {
                self: routes_1.extendedTabularResultUri(projectId, id)
            },
            paging: {
                offset: 0,
                count: values.length,
                total: values.length,
                next: null
            },
            values: values
        }
    };
}
function maybeWaitAndCreateResult(res, project, executionId, createResponseFn, orderBy, globalPollCount) {
    var _a = executionCache.get(executionId), execution = _a.execution, pollCounter = _a.pollCounter;
    var pollCount = execution.pollCount || globalPollCount;
    if (pollCounter < pollCount) {
        executionCache.set(executionId, { execution: execution, pollCounter: pollCounter + 1 });
        return res.status(HttpStatusCodes.ACCEPTED).send();
    }
    executionCache["delete"](executionId);
    if (execution.result === "empty") {
        // 'ok' | 'empty' | 'too large' | 'not computable'
        return res.status(HttpStatusCodes.NO_CONTENT).send();
    }
    if (execution.result === "too large") {
        return res.status(HttpStatusCodes.REQUEST_TOO_LONG).send();
    }
    if (execution.result === "not computable") {
        return res.status(HttpStatusCodes.BAD_REQUEST).send();
    }
    return res.status(HttpStatusCodes.OK).send(createResponseFn(project, execution, executionId, orderBy));
}
exports.executions = {
    register: function (app, project, config, idGenerator) {
        if (idGenerator === void 0) { idGenerator = lodash_1.uniqueId; }
        var orderBy;
        app.post("/gdc/internal/projects/" + project.project.meta.identifier + "/experimental/executions", function (req, res) {
            orderBy = req.body.execution.orderBy ? req.body.execution.orderBy[0] : null;
            var executionId = idGenerator();
            var response = createResponse(req.body, project, project.project.meta.identifier, executionId);
            return res.status(HttpStatusCodes.CREATED).send(response);
        });
        app.get("/gdc/internal/projects/" + project.project.meta.identifier + "/experimental/executions/:id", function (req, res) {
            return maybeWaitAndCreateResult(res, project, req.params.id, createResult, orderBy, config.pollCount);
        });
        app.get("/gdc/internal/projects/" + project.project.meta.identifier + "/experimental/executions/extendedResults/:id", function (req, res) {
            return maybeWaitAndCreateResult(res, project, req.params.id, lodash_1.partial(createExtendedResult, project.project.meta.identifier), orderBy, config.pollCount);
        });
        return app;
    }
};
//# sourceMappingURL=executions.js.map