"use strict";
exports.__esModule = true;
exports.log = void 0;
var lodash_1 = require("lodash");
var HttpStatusCodes = require("http-status-codes");
var ValidationError_1 = require("../ValidationError");
var logErrors = require("../../constants/ProjectLogErrors");
var errors = require("../errors/errors");
var areStrings = function (messages) { return messages.every(lodash_1.isString); };
function validate(request) {
    if (Object.keys(request).length !== 1) {
        throw new ValidationError_1.ValidationError(errors.moreThanOneRootElement());
    }
    if (!lodash_1.has(request, "logMessages")) {
        throw new ValidationError_1.ValidationError(errors.invalidElementKey("logMessages", Object.keys(request)[0]));
    }
    if (!Array.isArray(request.logMessages)) {
        throw new ValidationError_1.ValidationError(logErrors.INVALID_ELEMENT_TYPE);
    }
    if (!areStrings(request.logMessages)) {
        throw new ValidationError_1.ValidationError(logErrors.INVALID_TYPES_IN_ELEMENT);
    }
}
exports.log = {
    register: function (app, project) {
        app.post("/gdc/app/projects/" + project.project.meta.identifier + "/log", function (req, res) {
            var request = req.body;
            try {
                validate(request);
                res.status(HttpStatusCodes.NO_CONTENT).end();
            }
            catch (error) {
                if (error instanceof ValidationError_1.ValidationError) {
                    res.status(HttpStatusCodes.BAD_REQUEST).json(error.getValidationError());
                }
                else {
                    res.status(HttpStatusCodes.INTERNAL_SERVER_ERROR).json(error);
                }
            }
        });
        return app;
    }
};
//# sourceMappingURL=log.js.map