"use strict";
exports.__esModule = true;
exports.tokenExpiration = void 0;
// (C) 2007-2019 GoodData Corporation
var HttpStatusCodes = require("http-status-codes");
var lodash_1 = require("lodash");
var TokenHandler = /** @class */ (function () {
    function TokenHandler(isValidToken) {
        if (isValidToken === void 0) { isValidToken = true; }
        this.isValidToken = isValidToken;
    }
    TokenHandler.prototype.setTokenState = function (state) {
        this.isValidToken = state;
    };
    TokenHandler.prototype.isTokenValid = function () {
        return this.isValidToken;
    };
    return TokenHandler;
}());
function tokenExpiration(options) {
    var tokenHandler = new TokenHandler();
    var invalidateToken = lodash_1.throttle(function () { return tokenHandler.setTokenState(false); }, options.expirationTime, {
        leading: false
    });
    return function (req, res, next) {
        if (!req.path.match(/\/gdc/)) {
            return next();
        }
        // In case of request on the `/gdc/account/token` endpoint
        // token is marked as valid and 200 response is returned.
        var isTokenRequest = req.path.match(/\/gdc\/account\/token/);
        if (isTokenRequest) {
            tokenHandler.setTokenState(true);
            return res.status(HttpStatusCodes.OK).end();
        }
        if (tokenHandler.isTokenValid()) {
            invalidateToken();
            return next();
        }
        return res.status(HttpStatusCodes.UNAUTHORIZED).end();
    };
}
exports.tokenExpiration = tokenExpiration;
//# sourceMappingURL=tokenExpiration.js.map