import { VisualizationObject, VisualizationClass, ExtendedDateFilters } from "@gooddata/typings";
import { IAttribute } from "./Attribute";
import { IFact } from "./Fact";
import { IUser } from "./User";
import { IMetric } from "./Metric";
import { IProject } from "./Project";
import { IProjectPermissions } from "./ProjectPermissions";
import { IDateAttribute, IDateDataSet } from "./DateDataSet";
import { IElement } from "./Element";
import { IFeatureFlags } from "./FeatureFlags";
import { IConfigSettingItem } from "./ProjectConfigs";
import { IExecution } from "./Executions";
import { IAnalyticalDashboard } from "./AnalyticalDashboard";
import { IVisualizationWidget } from "./VisualizationWidget";
import { IKpi } from "./Kpi";
import { IFilterContext } from "./FilterContext";
import { Widget } from "./Widget";
import { IKpiAlert } from "./KpiAlert";
import { IAttributeDisplayForm } from "./AttributeDisplayForm";
import { ISchemaAfmExecution } from "../schema/model/SchemaAfmExecution";
import { ElementMap } from "../schema/builders/ElementsBuilder";
import { DateRangesMap } from "../schema/builders/DateRangesBuilder";
import { IEndPoints } from "./endPoints/EndPoints";
import { IBootstrap } from "./Bootstrap";
import { IStyleSettings } from "../model/StyleSetting";
import { IMockResponseConfig } from "../endpoints/helpers/mockRequest";
export declare type GroupType = "production" | "csv";
export interface IGroupMeta {
    identifier: string;
    type: GroupType;
}
export interface IProjectGroup {
    attributes: IAttribute[];
    metrics: IMetric[];
    facts: IFact[];
    dateDataSets: IDateDataSet[];
    meta: IGroupMeta;
}
export interface IOrganizedCatalog {
    items?: IMockResponseConfig[];
    groups?: IMockResponseConfig[];
    queryResponse?: IMockResponseConfig[];
    queryResult?: IMockResponseConfig[];
}
export interface IMockProject {
    project?: IProject;
    projectPermissions?: IProjectPermissions;
    users?: IUser[];
    groups?: IProjectGroup[];
    elements?: ElementMap;
    dateRanges?: DateRangesMap;
    featureFlags?: IFeatureFlags;
    projectConfigs?: IConfigSettingItem[];
    userConfigs?: IConfigSettingItem[];
    executions?: IExecution[];
    afmExecutions?: ISchemaAfmExecution[];
    endPoints?: IEndPoints;
    analyticalDashboards?: IAnalyticalDashboard[];
    visualizationObjects?: VisualizationObject.IVisualizationObject[];
    visualizationClasses?: VisualizationClass.IVisualizationClass[];
    visualizationWidgets?: IVisualizationWidget[];
    kpis?: IKpi[];
    filterContexts?: IFilterContext[];
    widgets?: Widget[];
    kpiAlerts?: IKpiAlert[];
    bootstrap?: IBootstrap;
    styleSettings?: IStyleSettings;
    dateFilterConfig?: ExtendedDateFilters.IDateFilterConfig;
    organizedCatalog?: IOrganizedCatalog;
}
export declare function getAttributes(project: IMockProject): IAttribute[];
export declare function getDateAttributes(project: IMockProject): IDateAttribute[];
export declare function getAttributeDisplayForms(project: IMockProject): IAttributeDisplayForm[];
export declare function getDateAttributeDisplayForms(project: IMockProject): IAttributeDisplayForm[];
export declare function getMetrics(project: IMockProject): IMetric[];
export declare function getFacts(project: IMockProject): IFact[];
export declare function getFilterContexts(project: IMockProject): IFilterContext[];
export declare function getDateDataSets(project: IMockProject): IDateDataSet[];
export declare function getDateDataSetDisplayForms(dateDataSet: IDateDataSet): IAttributeDisplayForm[];
export declare function getAllDateDataSetDisplayForms(project: IMockProject): IAttributeDisplayForm[];
export declare function getWidgets(project: IMockProject): Widget[];
export declare function getAnalyticalDashboards(project: IMockProject): IAnalyticalDashboard[];
export declare function getVisualizationObjects(project: IMockProject): VisualizationObject.IVisualizationObject[];
export declare function getVisualizationObject(project: IMockProject, uri: string): VisualizationObject.IVisualizationObject;
export declare function getVisualizationClasses(project: IMockProject): VisualizationClass.IVisualizationClass[];
export declare function getVisualizationWidgets(project: IMockProject): IVisualizationWidget[];
export declare function getDateFilterConfig(project: IMockProject): ExtendedDateFilters.IDateFilterConfig;
export declare function getAttributeByUri(project: IMockProject, uri: string): IAttribute;
export declare function getAttributeByTitle(project: IMockProject, title: string): IAttribute;
export declare function isKnownDisplayFormId(project: IMockProject, displayFormId: string): boolean;
export declare function isDateDisplayForm(project: IMockProject, displayForm: IAttributeDisplayForm): boolean;
export declare function isDateDisplayFormById(project: IMockProject, displayFormId: string): boolean;
export declare function getAttributeElementsByDisplayForm(project: IMockProject, displayFormId: string): IElement[];
