"use strict";
exports.__esModule = true;
exports.getAttributeElementsByDisplayForm = exports.isDateDisplayFormById = exports.isDateDisplayForm = exports.isKnownDisplayFormId = exports.getAttributeByTitle = exports.getAttributeByUri = exports.getDateFilterConfig = exports.getVisualizationWidgets = exports.getVisualizationClasses = exports.getVisualizationObject = exports.getVisualizationObjects = exports.getAnalyticalDashboards = exports.getWidgets = exports.getAllDateDataSetDisplayForms = exports.getDateDataSetDisplayForms = exports.getDateDataSets = exports.getFilterContexts = exports.getFacts = exports.getMetrics = exports.getDateAttributeDisplayForms = exports.getAttributeDisplayForms = exports.getDateAttributes = exports.getAttributes = void 0;
var lodash_1 = require("lodash");
function getAttributes(project) {
    return lodash_1.flatMap(project.groups, function (group) { return group.attributes; });
}
exports.getAttributes = getAttributes;
function getDateAttributes(project) {
    return lodash_1.flatMap(getDateDataSets(project), function (dateDataSet) { return dateDataSet.availableDateAttributes; });
}
exports.getDateAttributes = getDateAttributes;
function getAttributeDisplayForms(project) {
    var attributes = getAttributes(project);
    return lodash_1.flatMap(attributes, function (attribute) { return attribute.content.displayForms; });
}
exports.getAttributeDisplayForms = getAttributeDisplayForms;
function getDateAttributeDisplayForms(project) {
    return lodash_1.flatMap(getDateAttributes(project), function (dateAttribute) { return dateAttribute.displayForms; });
}
exports.getDateAttributeDisplayForms = getDateAttributeDisplayForms;
function getMetrics(project) {
    return lodash_1.flatMap(project.groups, function (group) { return group.metrics; });
}
exports.getMetrics = getMetrics;
function getFacts(project) {
    return lodash_1.flatMap(project.groups, function (group) { return group.facts; });
}
exports.getFacts = getFacts;
function getFilterContexts(project) {
    return project.filterContexts;
}
exports.getFilterContexts = getFilterContexts;
function getDateDataSets(project) {
    return lodash_1.flatMap(project.groups, function (group) { return group.dateDataSets; });
}
exports.getDateDataSets = getDateDataSets;
function getDateDataSetDisplayForms(dateDataSet) {
    return lodash_1.flatMap(dateDataSet.availableDateAttributes, function (attribute) { return attribute.displayForms; });
}
exports.getDateDataSetDisplayForms = getDateDataSetDisplayForms;
function getAllDateDataSetDisplayForms(project) {
    var dateDataSets = getDateDataSets(project);
    return lodash_1.flatMap(dateDataSets, getDateDataSetDisplayForms);
}
exports.getAllDateDataSetDisplayForms = getAllDateDataSetDisplayForms;
function getWidgets(project) {
    return project.widgets;
}
exports.getWidgets = getWidgets;
function getAnalyticalDashboards(project) {
    return project.analyticalDashboards;
}
exports.getAnalyticalDashboards = getAnalyticalDashboards;
function getVisualizationObjects(project) {
    return project.visualizationObjects;
}
exports.getVisualizationObjects = getVisualizationObjects;
function getVisualizationObject(project, uri) {
    return project.visualizationObjects
        ? project.visualizationObjects.find(function (obj) { return obj.meta.uri === uri; })
        : undefined;
}
exports.getVisualizationObject = getVisualizationObject;
function getVisualizationClasses(project) {
    return project.visualizationClasses;
}
exports.getVisualizationClasses = getVisualizationClasses;
function getVisualizationWidgets(project) {
    var widgets = project.widgets
        ? project.widgets.filter(function (widget) { return widget.meta.category === "visualizationWidget"; })
        : [];
    return widgets;
}
exports.getVisualizationWidgets = getVisualizationWidgets;
function getDateFilterConfig(project) {
    return project.dateFilterConfig;
}
exports.getDateFilterConfig = getDateFilterConfig;
function getAttributeByUri(project, uri) {
    return getAttributes(project).find(function (attribute) { return attribute.meta.uri === uri; });
}
exports.getAttributeByUri = getAttributeByUri;
function getAttributeByTitle(project, title) {
    return getAttributes(project).find(function (attribute) { return attribute.meta.title === title; });
}
exports.getAttributeByTitle = getAttributeByTitle;
function isKnownDisplayFormId(project, displayFormId) {
    var displayForms = lodash_1.flatMap(getAttributes(project), function (attribute) { return attribute.content.displayForms; });
    return displayForms.some(function (displayForm) { return displayForm.meta.identifier === displayFormId; });
}
exports.isKnownDisplayFormId = isKnownDisplayFormId;
function isDateDisplayForm(project, displayForm) {
    return getAllDateDataSetDisplayForms(project).some(function (dateDisplayForm) { return dateDisplayForm.meta.identifier === displayForm.meta.identifier; });
}
exports.isDateDisplayForm = isDateDisplayForm;
function isDateDisplayFormById(project, displayFormId) {
    return getAllDateDataSetDisplayForms(project).some(function (dateDisplayForm) { return dateDisplayForm.meta.identifier === displayFormId; });
}
exports.isDateDisplayFormById = isDateDisplayFormById;
function getAttributeElementsByDisplayForm(project, displayFormId) {
    var displayForms = getAttributeDisplayForms(project);
    var displayForm = displayForms.find(function (df) { return df.meta.identifier === displayFormId; });
    if (!displayForm) {
        return [];
    }
    var attribute = getAttributes(project).find(function (attr) { return attr.meta.uri === displayForm.content.formOf; });
    return attribute ? project.elements.get(attribute.meta.identifier) : [];
}
exports.getAttributeElementsByDisplayForm = getAttributeElementsByDisplayForm;
//# sourceMappingURL=MockProject.js.map