"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createAnalyticalDashboards = exports.createAnalyticalDashboard = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var Project_1 = require("../../model/Project");
var Widget_1 = require("../../model/Widget");
var routes_1 = require("../../route/routes");
var string_1 = require("../../utils/string");
var objects_1 = require("../helpers/objects");
var WidgetBuilder_1 = require("./WidgetBuilder");
var ProjectBuilder_1 = require("./ProjectBuilder");
var LayoutBuilder_1 = require("./LayoutBuilder");
var DateFilterConfigBuilder_1 = require("./DateFilterConfigBuilder");
var createDashboardIdentifier = function (dashboard) {
    return "dash." + string_1.webalize(dashboard.title);
};
function getWidgetUriMapper(widgets) {
    var uriHash = widgets.reduce(function (hash, widget) {
        var _a;
        return (__assign(__assign({}, hash), (_a = {}, _a[Widget_1.getWidgetId(widget)] = Widget_1.getWidgetUri(widget), _a)));
    }, {});
    return function (widget) { return uriHash[objects_1.ensureIdentifier(widget, string_1.webalize(widget.title))]; };
}
function createAnalyticalDashboard(project, widgets, analyticalDashboard) {
    var _a = analyticalDashboard.widgets, dashboardWidgets = _a === void 0 ? [] : _a, title = analyticalDashboard.title;
    var identifier = objects_1.ensureIdentifier(analyticalDashboard, createDashboardIdentifier(analyticalDashboard));
    var uri = routes_1.objectUri(Project_1.getProjectId(project), identifier);
    var filterContextTitle = lodash_1.get(analyticalDashboard, "filterContext.title");
    var layout = lodash_1.get(analyticalDashboard, "layout");
    var layoutProp = layout ? { layout: LayoutBuilder_1.createLayout(layout, project) } : {};
    var extendedDateFilters = lodash_1.get(analyticalDashboard, "dateFilterConfig");
    var edfProp = extendedDateFilters
        ? { dateFilterConfig: DateFilterConfigBuilder_1.createDateFilterDashboardConfig(extendedDateFilters) }
        : {};
    return {
        meta: {
            category: "analyticalDashboard",
            identifier: identifier,
            title: title,
            uri: uri
        },
        content: __assign(__assign({ filterContext: filterContextTitle
                ? routes_1.objectUri(Project_1.getProjectId(project), string_1.webalize(filterContextTitle))
                : "", widgets: dashboardWidgets.map(getWidgetUriMapper(widgets)) }, layoutProp), edfProp)
    };
}
exports.createAnalyticalDashboard = createAnalyticalDashboard;
function createAnalyticalDashboards(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var widgets = WidgetBuilder_1.createWidgets(schema);
    return (schema.analyticalDashboards || []).map(lodash_1.partial(createAnalyticalDashboard, project, widgets));
}
exports.createAnalyticalDashboards = createAnalyticalDashboards;
//# sourceMappingURL=AnalyticalDashboardBuilder.js.map