"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.createAttribute = exports.createAttributeIdentifier = void 0;
// (C) 2007-2021 GoodData Corporation
var Project_1 = require("../../model/Project");
var routes_1 = require("../../route/routes");
var objects_1 = require("../helpers/objects");
var string_1 = require("../../utils/string");
exports.createAttributeIdentifier = function (attribute) { return "attr." + string_1.webalize(attribute.title); };
function getDisplayFormType(type) {
    switch (type) {
        case "hyperlink":
            return "GDC.link";
        case "geoPin":
            return "GDC.geo.pin";
        case "text":
        default:
            return undefined;
    }
}
function createDefaultDisplayForm(project, attributeId, attribute) {
    var projectId = project.meta.identifier;
    var displayFormId = attributeId + ".df";
    var type = attribute.defaultDisplayFormType;
    return createDisplayForm(projectId, type, displayFormId, attributeId, attribute.title);
}
function createAdditionalDisplayForms(projectId, attributeId, attribute) {
    var additionalDisplayForms = attribute.additionalDisplayForms || [];
    return additionalDisplayForms.map(function (displayForm) {
        var title = attribute.title + " (" + displayForm.identifier + ")";
        var type = getDisplayFormType(displayForm.type);
        return createDisplayForm(projectId, type, displayForm.identifier, attributeId, title);
    });
}
function createDisplayForm(projectId, type, displayFormId, attributeId, title) {
    return {
        content: {
            expression: "[" + routes_1.objectUri(projectId, "123") + "]",
            formOf: routes_1.objectUri(projectId, attributeId),
            type: type
        },
        links: {
            elements: routes_1.elementsUri(projectId, attributeId)
        },
        meta: {
            category: "attributeDisplayForm",
            identifier: displayFormId,
            title: title,
            uri: routes_1.objectUri(projectId, displayFormId)
        }
    };
}
function createAttribute(project, attribute) {
    var identifier = objects_1.ensureIdentifier(attribute, exports.createAttributeIdentifier(attribute));
    var projectId = Project_1.getProjectId(project);
    var drillDown = attribute.drillDownStepAttributeDF
        ? {
            drillDownStepAttributeDF: routes_1.objectUri(projectId, attribute.drillDownStepAttributeDF)
        }
        : {};
    return {
        content: __assign({ displayForms: __spreadArrays([
                createDefaultDisplayForm(project, identifier, attribute)
            ], createAdditionalDisplayForms(projectId, identifier, attribute)) }, drillDown),
        meta: {
            category: "attribute",
            title: attribute.title,
            identifier: identifier,
            uri: routes_1.objectUri(projectId, identifier),
            tags: attribute.tags
        }
    };
}
exports.createAttribute = createAttribute;
//# sourceMappingURL=AttributeBuilder.js.map