"use strict";
exports.__esModule = true;
exports.createDateDataSet = void 0;
// (C) 2007-2021 GoodData Corporation
var lodash_1 = require("lodash");
var string_1 = require("../../utils/string");
var routes_1 = require("../../route/routes");
function createDateDataSet(project, dateDataSet) {
    var types = [
        "date",
        "day_in_week",
        "day_in_month",
        "day_in_quarter",
        "day_in_year",
        "week_us",
        "week_in_quarter",
        "week_in_year",
        "month",
        "month_in_quarter",
        "month_in_year",
        "quarter",
        "quarter_in_year",
        "year",
    ];
    var titles = [
        "Date",
        "Day of Week",
        "Day of Month",
        "Day of Quarter",
        "Day of Year",
        "Week (Sun-Sat)/Year",
        "Week of Quarter",
        "Week of Year",
        "Month/Year",
        "Month of Quarter",
        "Month of Year",
        "Quarter/Year",
        "Quarter of Year",
        "Year",
    ];
    var projectId = project.meta.identifier;
    var dateDataSetId = string_1.webalize(dateDataSet.title);
    function createDateAttributeFragment(type, index) {
        var identifier = "attr." + dateDataSetId + "." + type;
        var attrUri = routes_1.objectUri(projectId, identifier);
        var defaultDisplayFormUri = routes_1.objectUri(projectId, identifier + ".df");
        var secondaryDisplayFormUri = routes_1.objectUri(projectId, identifier + ".df.shortened");
        return {
            type: "GDC.time." + type,
            meta: {
                category: "attribute",
                title: titles[index] + " (" + lodash_1.capitalize(dateDataSetId) + ")",
                identifier: identifier,
                isProduction: true,
                uri: attrUri
            },
            displayForms: [
                {
                    content: {
                        expression: "",
                        formOf: routes_1.objectUri(projectId, identifier)
                    },
                    links: {
                        elements: defaultDisplayFormUri + "/elements"
                    },
                    meta: {
                        category: "attributeDisplayForm",
                        title: type + " df",
                        identifier: identifier + ".df",
                        isProduction: true,
                        uri: defaultDisplayFormUri
                    }
                },
                {
                    content: {
                        expression: "",
                        formOf: routes_1.objectUri(projectId, identifier)
                    },
                    links: {
                        elements: secondaryDisplayFormUri + "/elements"
                    },
                    meta: {
                        category: "attributeDisplayForm",
                        title: type + " df shortened",
                        identifier: identifier + ".df.shortened",
                        isProduction: true,
                        uri: secondaryDisplayFormUri
                    }
                },
            ]
        };
    }
    return {
        availableDateAttributes: types.map(createDateAttributeFragment),
        meta: {
            category: "dateDataSet",
            title: dateDataSet.title,
            identifier: dateDataSetId + ".dataset",
            uri: routes_1.objectUri(projectId, dateDataSetId + ".dataset"),
            tags: dateDataSet.tags
        },
        relevance: 0
    };
}
exports.createDateDataSet = createDateDataSet;
//# sourceMappingURL=DateDataSetBuilder.js.map