"use strict";
exports.__esModule = true;
exports.createDateRangesMap = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var objects_1 = require("../helpers/objects");
var ProjectBuilder_1 = require("./ProjectBuilder");
var DateDataSetBuilder_1 = require("./DateDataSetBuilder");
var DateRange_1 = require("../../utils/model/DateRange");
function createDateRangesMap(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var groups = objects_1.ensureItems(schema.groups);
    var dateDataSets = lodash_1.flatMap(groups, function (group) { return objects_1.ensureItems(group.dateDataSets); });
    return dateDataSets.reduce(function (map, dateDataSetSchema) {
        var dateDataSet = DateDataSetBuilder_1.createDateDataSet(project, dateDataSetSchema);
        var startDate = dateDataSetSchema.startDate, finalDate = dateDataSetSchema.finalDate;
        return map.set(dateDataSet.meta.identifier, new DateRange_1.DateRange(startDate, finalDate));
    }, new Map());
}
exports.createDateRangesMap = createDateRangesMap;
//# sourceMappingURL=DateRangesBuilder.js.map