"use strict";
exports.__esModule = true;
exports.createDrillDefinitions = void 0;
var SchemaDrillDefinition_1 = require("../model/SchemaDrillDefinition");
var Project_1 = require("../../model/Project");
var routes_1 = require("../../route/routes");
var invariant = require("invariant");
function createDrillDefinitions(drillDefinitions, project) {
    if (drillDefinitions) {
        return drillDefinitions.map(function (drillDefinition) { return createDrillDefinition(drillDefinition, project); });
    }
}
exports.createDrillDefinitions = createDrillDefinitions;
function createDrillToVisualizationDefinition(drillDefinition, project) {
    var drillToVisualization = drillDefinition.drillToVisualization;
    return {
        drillToVisualization: {
            target: drillToVisualization.target,
            from: {
                drillFromMeasure: {
                    localIdentifier: drillToVisualization.from
                }
            },
            toVisualization: {
                uri: routes_1.objectUri(Project_1.getProjectId(project), drillToVisualization.toVisualization)
            }
        }
    };
}
function createDrillToDashboardDefinition(drillDefinition) {
    var drillToDashboard = drillDefinition.drillToDashboard;
    return {
        drillToDashboard: {
            target: drillToDashboard.target,
            from: {
                drillFromMeasure: {
                    localIdentifier: drillToDashboard.from
                }
            },
            toDashboard: drillToDashboard.toDashboard
        }
    };
}
function createDrillToCustomUrlDefinition(drillDefinition) {
    var _a = drillDefinition.drillToCustomUrl, target = _a.target, from = _a.from, customUrl = _a.customUrl;
    return {
        drillToCustomUrl: {
            target: target,
            from: {
                drillFromMeasure: {
                    localIdentifier: from
                }
            },
            customUrl: customUrl
        }
    };
}
function createDrillToAttributeUrlDefinition(drillDefinition) {
    var _a = drillDefinition.drillToAttributeUrl, target = _a.target, from = _a.from, insightAttributeDisplayForm = _a.insightAttributeDisplayForm, drillToAttributeDisplayForm = _a.drillToAttributeDisplayForm;
    return {
        drillToAttributeUrl: {
            target: target,
            from: {
                drillFromMeasure: {
                    localIdentifier: from
                }
            },
            insightAttributeDisplayForm: {
                uri: insightAttributeDisplayForm
            },
            drillToAttributeDisplayForm: {
                uri: drillToAttributeDisplayForm
            }
        }
    };
}
function createDrillDefinition(drillDefinition, project) {
    if (SchemaDrillDefinition_1.isSchemaDrillToVisualization(drillDefinition)) {
        return createDrillToVisualizationDefinition(drillDefinition, project);
    }
    else if (SchemaDrillDefinition_1.isSchemaDrillToDashboard(drillDefinition)) {
        return createDrillToDashboardDefinition(drillDefinition);
    }
    else if (SchemaDrillDefinition_1.isSchemaDrillToCustomUrl(drillDefinition)) {
        return createDrillToCustomUrlDefinition(drillDefinition);
    }
    else if (SchemaDrillDefinition_1.isSchemaDrillToAttributeUrl(drillDefinition)) {
        return createDrillToAttributeUrlDefinition(drillDefinition);
    }
    invariant(drillDefinition, "Unsupported drillDefinition schema.");
}
//# sourceMappingURL=DrillDefinitionBuilder.js.map