"use strict";
exports.__esModule = true;
exports.createKpiAlerts = void 0;
var lodash_1 = require("lodash");
var objects_1 = require("../helpers/objects");
var routes_1 = require("../../route/routes");
var Project_1 = require("../../model/Project");
var string_1 = require("../../utils/string");
var UserBuilder_1 = require("./UserBuilder");
var ProjectBuilder_1 = require("./ProjectBuilder");
var createKpiAlertIdentifier = function (kpiAlert) {
    return "alert." + string_1.webalize(kpiAlert.kpiId) + "." + string_1.webalize(kpiAlert.dashboardId);
};
function createKpiAlert(project, users, kpiAlert) {
    var author = routes_1.profileUri(kpiAlert.author || lodash_1.first(users).identifier);
    var dashboardUri = routes_1.objectUri(Project_1.getProjectId(project), kpiAlert.dashboardId);
    var projectId = Project_1.getProjectId(project);
    var identifier = objects_1.ensureIdentifier(kpiAlert, createKpiAlertIdentifier(kpiAlert));
    var filterContextTitle = lodash_1.get(kpiAlert, "filterContext.title");
    var kpiId = kpiAlert.kpiId, threshold = kpiAlert.threshold, isTriggered = kpiAlert.isTriggered, whenTriggered = kpiAlert.whenTriggered;
    return {
        content: {
            kpi: routes_1.objectUri(projectId, kpiId),
            dashboard: dashboardUri,
            threshold: threshold,
            isTriggered: isTriggered,
            whenTriggered: whenTriggered,
            filterContext: filterContextTitle ? routes_1.objectUri(projectId, string_1.webalize(filterContextTitle)) : ""
        },
        meta: {
            author: author,
            category: "kpiAlert",
            identifier: identifier,
            title: "kpi alert",
            uri: routes_1.objectUri(projectId, identifier)
        }
    };
}
function createKpiAlerts(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var users = UserBuilder_1.createUsers(schema);
    return objects_1.ensureItems(schema.kpiAlerts).map(lodash_1.partial(createKpiAlert, project, users));
}
exports.createKpiAlerts = createKpiAlerts;
//# sourceMappingURL=KpiAlertBuilder.js.map