"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createKpis = exports.createKpi = void 0;
// (C) 2007-2019 GoodData Corporation
var lodash_1 = require("lodash");
var Project_1 = require("../../model/Project");
var routes_1 = require("../../route/routes");
var string_1 = require("../../utils/string");
var ProjectBuilder_1 = require("./ProjectBuilder");
var UserBuilder_1 = require("./UserBuilder");
function getContentBase(project, kpi) {
    return __assign({ metric: routes_1.objectUri(Project_1.getProjectId(project), kpi.metricIdentifier), ignoreDashboardFilters: [] }, (kpi.dateDataSetUri && { dateDataSet: kpi.dateDataSetUri }));
}
function getContentWithComparison(project, kpi) {
    return __assign(__assign({}, getContentBase(project, kpi)), { comparisonType: kpi.comparisonType, comparisonDirection: kpi.comparisonDirection });
}
function getContentWithoutComparison(project, kpi) {
    return __assign(__assign({}, getContentBase(project, kpi)), { comparisonType: "none" });
}
function getDashboardKpis(analyticalDashboard) {
    var widgets = analyticalDashboard.widgets
        ? analyticalDashboard.widgets.filter(function (widget) { return lodash_1.has(widget, "metricIdentifier"); })
        : [];
    return widgets;
}
function createKpi(project, userId, kpi) {
    var identifier = kpi.identifier || string_1.webalize(kpi.title);
    return {
        content: kpi.comparisonDirection
            ? getContentWithComparison(project, kpi)
            : getContentWithoutComparison(project, kpi),
        meta: {
            author: routes_1.profileUri(userId),
            category: "kpi",
            deprecated: false,
            identifier: identifier,
            locked: false,
            title: kpi.title,
            unlisted: false,
            uri: routes_1.objectUri(Project_1.getProjectId(project), identifier)
        }
    };
}
exports.createKpi = createKpi;
function createKpis(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var users = UserBuilder_1.createUsers(schema);
    var kpis = lodash_1.flatMap(schema.analyticalDashboards || [], function (analyticalDashboard) { return getDashboardKpis(analyticalDashboard) || []; });
    return kpis.map(function (kpi) { return createKpi(project, users[0].identifier, kpi); });
}
exports.createKpis = createKpis;
//# sourceMappingURL=KpiBuilder.js.map