"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createLayoutContent = exports.createFluidLayout = exports.createLayout = void 0;
var Project_1 = require("../../model/Project");
var routes_1 = require("../../route/routes");
var layoutUtils_1 = require("../../utils/layoutUtils");
function createLayout(layoutSchema, project) {
    var layout;
    if (layoutUtils_1.isFluidLayout(layoutSchema)) {
        return (layout = createFluidLayout(layoutSchema, project));
    }
    return layout;
}
exports.createLayout = createLayout;
function createFluidLayout(fluidLayoutSchema, project) {
    var rows = fluidLayoutSchema.fluidLayout.rows.map(function (row) { return createFluidRow(row, project); });
    return {
        fluidLayout: {
            rows: rows,
            size: fluidLayoutSchema.fluidLayout.size,
            style: fluidLayoutSchema.fluidLayout.style
        }
    };
}
exports.createFluidLayout = createFluidLayout;
function createLayoutContent(layoutContentSchema, project) {
    if (layoutUtils_1.isPersistedLayoutWidget(layoutContentSchema)) {
        return createPersistedLayoutWidget(layoutContentSchema, project);
    }
    return null;
}
exports.createLayoutContent = createLayoutContent;
function createFluidRow(fluidRowSchema, project) {
    var columns = fluidRowSchema.columns.map(function (column) { return createFluidColumn(column, project); });
    return {
        columns: columns,
        style: fluidRowSchema.style,
        header: fluidRowSchema.header
    };
}
function createFluidColumn(fluidColumnSchema, project) {
    var content = fluidColumnSchema.content
        ? { content: createLayoutContent(fluidColumnSchema.content, project) }
        : {};
    return __assign({ size: fluidColumnSchema.size, style: fluidColumnSchema.style }, content);
}
function createPersistedLayoutWidget(widgetSchema, project) {
    return {
        widget: {
            qualifier: {
                uri: routes_1.objectUri(Project_1.getProjectId(project), widgetSchema.widget.identifier)
            }
        }
    };
}
//# sourceMappingURL=LayoutBuilder.js.map