"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createOrganizedCatalog = void 0;
var ProjectBuilder_1 = require("./ProjectBuilder");
var Project_1 = require("../../model/Project");
var routes_1 = require("../../route/routes");
var lodash_1 = require("lodash");
function createOrganizedCatalog(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var projectId = Project_1.getProjectId(project);
    if (schema.organizedCatalog === undefined) {
        return;
    }
    var transformedSchema = lodash_1.cloneDeep(schema.organizedCatalog);
    transformedSchema.items = transformItemsMocks(schema.organizedCatalog.items || [], projectId);
    transformedSchema.queryResponse = transformQueryMocks(schema.organizedCatalog.queryResponse || [], projectId);
    transformedSchema.groups = transformGroupsMocks(schema.organizedCatalog.groups || []);
    return transformedSchema;
}
exports.createOrganizedCatalog = createOrganizedCatalog;
var transformGroupsMocks = function (originalGroupsMocks) {
    return originalGroupsMocks.map(function (mockResponseConfig) {
        var transformedMock = lodash_1.cloneDeep(mockResponseConfig);
        transformedMock.response = {
            catalogGroups: mockResponseConfig.response
        };
        return transformedMock;
    });
};
var transformItemsMocks = function (originalItemsMocks, projectId) {
    return originalItemsMocks.map(function (mockResponseConfig) {
        var transformedMock = lodash_1.cloneDeep(mockResponseConfig);
        var transformedItems = mockResponseConfig.response.map(function (item) { return (__assign(__assign({}, item), { links: getLinks(projectId, item) })); });
        transformedMock.response = {
            catalogItems: {
                items: transformedItems,
                paging: {
                    count: transformedItems.length,
                    offset: parseInt(mockResponseConfig.query.offset, 10)
                }
            }
        };
        return transformedMock;
    });
};
var transformQueryMocks = function (originalQueryMocks, projectId) {
    return originalQueryMocks.map(function (mockResponseConfig) {
        var transformedMock = lodash_1.cloneDeep(mockResponseConfig);
        transformedMock.response = {
            catalogAvailableItems: {
                items: mockResponseConfig.response.map(function (itemIdentifier) {
                    return routes_1.objectUri(projectId, itemIdentifier);
                })
            }
        };
        transformedMock.body.catalogQueryRequest.bucketItems = mockResponseConfig.body.catalogQueryRequest.bucketItems.map(function (itemIdentifier) { return ({ uri: routes_1.objectUri(projectId, itemIdentifier) }); });
        return transformedMock;
    });
};
var getLinks = function (projectId, item) {
    return item.type === "attribute"
        ? {
            defaultDisplayForm: routes_1.objectUri(projectId, item.identifier + ".df"),
            self: routes_1.objectUri(projectId, item.identifier)
        }
        : {
            self: routes_1.objectUri(projectId, item.identifier)
        };
};
//# sourceMappingURL=OrganizedCatalog.js.map