"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createVisualizationClasses = void 0;
var lodash_1 = require("lodash");
var objects_1 = require("../helpers/objects");
var routes_1 = require("../../route/routes");
var Project_1 = require("../../model/Project");
var string_1 = require("../../utils/string");
var UserBuilder_1 = require("./UserBuilder");
var ProjectBuilder_1 = require("./ProjectBuilder");
function createVisualizationClass(project, users, visualizationClass) {
    var author = routes_1.profileUri(visualizationClass.author || lodash_1.first(users).identifier);
    var identifier = objects_1.ensureIdentifier(visualizationClass, string_1.webalize(visualizationClass.title));
    var deprecated = visualizationClass.deprecated ? { deprecated: visualizationClass.deprecated } : {};
    return {
        content: {
            url: visualizationClass.url,
            icon: visualizationClass.url,
            iconSelected: visualizationClass.url + ".selected",
            checksum: "local"
        },
        meta: __assign(__assign({ author: author, category: "visualizationClass" }, deprecated), { identifier: identifier, title: visualizationClass.title, uri: routes_1.objectUri(Project_1.getProjectId(project), identifier) })
    };
}
function createVisualizationClasses(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var users = UserBuilder_1.createUsers(schema);
    return objects_1.ensureItems(schema.visualizationClasses).map(lodash_1.partial(createVisualizationClass, project, users));
}
exports.createVisualizationClasses = createVisualizationClasses;
//# sourceMappingURL=VisualizationClassBuilder.js.map