"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.createVisualizationObjects = void 0;
var lodash_1 = require("lodash");
var objects_1 = require("../helpers/objects");
var routes_1 = require("../../route/routes");
var Project_1 = require("../../model/Project");
var string_1 = require("../../utils/string");
var uuid_1 = require("uuid");
var invariant = require("invariant");
var UserBuilder_1 = require("./UserBuilder");
var ProjectBuilder_1 = require("./ProjectBuilder");
var VisualizationClassBuilder_1 = require("./VisualizationClassBuilder");
var DATE_FILTER = "date";
var ATTRIBUTE_FILTER = "attribute";
var MEASURE_VALUE_COMPARISON_FILTER = "measureComparison";
var MEASURE_VALUE_RANGE_FILTER = "measureRange";
var RANKING_FILTER = "ranking";
function createBucket(project, bucketSchema) {
    var localIdentifier = bucketSchema.localIdentifier, items = bucketSchema.items, totals = bucketSchema.totals;
    return __assign({ localIdentifier: localIdentifier, items: items.map(function (item) { return createBucketItem(project, item); }) }, createVisualizationTotals(totals));
}
function isVisualizationAttribute(item) {
    return item.displayForm !== undefined;
}
function toUriQualifier(uri) {
    return {
        uri: uri
    };
}
function createBucketItem(project, item) {
    if (isVisualizationAttribute(item)) {
        return createVisualizationAttribute(project, item);
    }
    return createMeasure(project, item);
}
function createVisualizationTotals(totals) {
    if (totals === undefined || totals === null) {
        return {};
    }
    return {
        totals: totals.map(function (total) {
            var type = total.type, measureIdentifier = total.measureIdentifier, attributeIdentifier = total.attributeIdentifier;
            return {
                type: type,
                measureIdentifier: measureIdentifier,
                attributeIdentifier: attributeIdentifier
            };
        })
    };
}
function createVisualizationAttribute(project, attribute) {
    var localIdentifier = attribute.localIdentifier ? attribute.localIdentifier : uuid_1.v4();
    return {
        visualizationAttribute: {
            localIdentifier: localIdentifier,
            displayForm: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), attribute.displayForm))
        }
    };
}
function getMeasureBase(measure, definition) {
    var localIdentifier = measure.localIdentifier ? measure.localIdentifier : uuid_1.v4();
    var optionalProps = {};
    if (measure.title !== undefined) {
        optionalProps.title = measure.title;
    }
    if (measure.alias !== undefined) {
        optionalProps.alias = measure.alias;
    }
    if (measure.format !== undefined) {
        optionalProps.format = measure.format;
    }
    return {
        measure: __assign({ definition: definition,
            localIdentifier: localIdentifier }, optionalProps)
    };
}
function createOrdinaryMeasure(project, measure) {
    return getMeasureBase(measure, createMeasureDefinition(project, measure));
}
function createDerivedMeasure(project, measure) {
    return getMeasureBase(measure, createDerivedMeasureDefinition(project, measure));
}
function createArithmeticMeasure(measure) {
    return getMeasureBase(measure, createArithmeticMeasureDefinition(measure));
}
function measureIsDerived(measure) {
    return measure.isDerived;
}
function measureIsArithmetic(measure) {
    return measure.isArithmetic;
}
function createMeasure(project, measure) {
    if (measureIsDerived(measure)) {
        return createDerivedMeasure(project, measure);
    }
    else if (measureIsArithmetic(measure)) {
        return createArithmeticMeasure(measure);
    }
    return createOrdinaryMeasure(project, measure);
}
function createMeasureDefinition(project, measure) {
    var identifier = measure.identifier, aggregation = measure.aggregation, computeRatio = measure.computeRatio;
    var definition = {
        item: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), identifier)),
        aggregation: aggregation,
        computeRatio: computeRatio
    };
    var definitionFilters = measure.filters
        ? {
            filters: measure.filters.map(lodash_1.partial(createVisualizationFilter, project))
        }
        : {};
    return {
        measureDefinition: __assign(__assign({}, definition), definitionFilters)
    };
}
function createDerivedMeasureDefinition(project, measure) {
    if (measure.popAttribute !== undefined) {
        return createPoPMeasureDefinition(project, measure);
    }
    else if (measure.previousPeriod !== undefined) {
        return createPreviousPeriodMeasureDefinition(project, measure);
    }
    else {
        throw Error("The derived measure type is not supported! " + JSON.stringify(measure));
    }
}
function createPoPMeasureDefinition(project, measure) {
    var measureIdentifier = measure.measureIdentifier, popAttribute = measure.popAttribute;
    return {
        popMeasureDefinition: {
            measureIdentifier: measureIdentifier,
            popAttribute: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), popAttribute))
        }
    };
}
function createPreviousPeriodMeasureDefinition(project, measure) {
    var measureIdentifier = measure.measureIdentifier, previousPeriod = measure.previousPeriod;
    return {
        previousPeriodMeasure: {
            measureIdentifier: measureIdentifier,
            dateDataSets: previousPeriod.dateDataSets.map(function (dateDataSet) { return ({
                dataSet: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), dateDataSet.dataSet)),
                periodsAgo: dateDataSet.periodsAgo
            }); })
        }
    };
}
function createArithmeticMeasureDefinition(measure) {
    var operator = measure.operator, measureIdentifiers = measure.measureIdentifiers;
    return {
        arithmeticMeasure: {
            measureIdentifiers: measureIdentifiers,
            operator: operator
        }
    };
}
function createVisualizationDateFilter(project, filter) {
    return filter.relative
        ? {
            relativeDateFilter: {
                from: Number(filter.from),
                to: Number(filter.to),
                granularity: filter.granularity,
                dataSet: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), filter.dataset))
            }
        }
        : {
            absoluteDateFilter: {
                from: String(filter.from),
                to: String(filter.to),
                dataSet: toUriQualifier(routes_1.objectUri(Project_1.getProjectId(project), filter.dataset))
            }
        };
}
function createVisualizationAttributeFilter(project, filter) {
    var projectId = Project_1.getProjectId(project);
    return filter.negative
        ? {
            negativeAttributeFilter: {
                displayForm: toUriQualifier(routes_1.objectUri(projectId, filter.identifier + ".df")),
                notIn: filter.elements.map(function (id) {
                    return routes_1.objectUri(projectId, filter.identifier + "/elements?id=" + id);
                })
            }
        }
        : {
            positiveAttributeFilter: {
                displayForm: toUriQualifier(routes_1.objectUri(projectId, filter.identifier + ".df")),
                "in": filter.elements.map(function (id) {
                    return routes_1.objectUri(projectId, filter.identifier + "/elements?id=" + id);
                })
            }
        };
}
function asNumber(value) {
    return lodash_1.toNumber(value);
}
function createTreatAsNumberProp(treatNullValuesAs) {
    return treatNullValuesAs === undefined ? {} : { treatNullValuesAs: treatNullValuesAs };
}
function createMeasureValueComparisonFilter(filter) {
    return {
        measureValueFilter: {
            measure: {
                localIdentifier: filter.identifier
            },
            condition: {
                comparison: __assign({ operator: filter.operator, value: asNumber(filter.value) }, createTreatAsNumberProp(filter.treatNullValuesAs))
            }
        }
    };
}
function createMeasureValueRangeFilter(filter) {
    return {
        measureValueFilter: {
            measure: {
                localIdentifier: filter.identifier
            },
            condition: {
                range: __assign({ operator: filter.operator, from: asNumber(filter.from), to: asNumber(filter.to) }, createTreatAsNumberProp(filter.treatNullValuesAs))
            }
        }
    };
}
function createRankingFilter(filter) {
    var attributesProp = filter.attributes
        ? { attributes: filter.attributes.map(function (localIdentifier) { return ({ localIdentifier: localIdentifier }); }) }
        : {};
    return {
        rankingFilter: __assign(__assign({ measures: [{ localIdentifier: filter.identifier }] }, attributesProp), { operator: filter.operator, value: asNumber(filter.value) })
    };
}
function createVisualizationFilter(project, filter) {
    switch (filter.type) {
        case DATE_FILTER:
            return createVisualizationDateFilter(project, filter);
        case ATTRIBUTE_FILTER:
        default:
            return createVisualizationAttributeFilter(project, filter);
    }
}
function createVisualizationExtendedFilter(project, filter) {
    switch (filter.type) {
        case MEASURE_VALUE_COMPARISON_FILTER:
            return createMeasureValueComparisonFilter(filter);
        case MEASURE_VALUE_RANGE_FILTER:
            return createMeasureValueRangeFilter(filter);
        case RANKING_FILTER:
            return createRankingFilter(filter);
        default:
            return createVisualizationFilter(project, filter);
    }
}
function createVisualizationObject(project, users, visualizationClasses, visualizationObject) {
    var author = routes_1.profileUri(visualizationObject.author || lodash_1.first(users).identifier);
    var identifier = visualizationObject.identifier || string_1.webalize(visualizationObject.title);
    var visualizationObjectType = visualizationObject.type;
    // find visualization class which has the same url to be able to link it
    var visualizationClass = visualizationClasses.find(function (visualizationClass) { return visualizationClass.content.url === visualizationObjectType; });
    invariant(visualizationClass, "VisualizationClass with url " + visualizationObjectType + " not found.");
    var properties = lodash_1.get(visualizationObject, "properties", "");
    var propertiesProp = properties.length > 0 ? { properties: properties } : {};
    var filtersProp = visualizationObject.filters
        ? {
            filters: visualizationObject.filters.map(lodash_1.partial(createVisualizationExtendedFilter, project))
        }
        : {};
    return {
        content: __assign(__assign({ visualizationClass: toUriQualifier(lodash_1.get(visualizationClass, "meta.uri")), buckets: __spreadArrays(objects_1.ensureItems(visualizationObject.buckets).map(lodash_1.partial(createBucket, project))) }, propertiesProp), filtersProp),
        meta: {
            author: author,
            category: "visualizationObject",
            identifier: identifier,
            title: visualizationObject.title,
            uri: routes_1.objectUri(Project_1.getProjectId(project), identifier)
        }
    };
}
function createVisualizationObjects(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var users = UserBuilder_1.createUsers(schema);
    var visualizationClasses = VisualizationClassBuilder_1.createVisualizationClasses(schema);
    return objects_1.ensureItems(schema.visualizationObjects).map(lodash_1.partial(createVisualizationObject, project, users, visualizationClasses));
}
exports.createVisualizationObjects = createVisualizationObjects;
//# sourceMappingURL=VisualizationObjectBuilder.js.map