"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.createVisualizationWidgets = exports.createVisualizationWidget = void 0;
// (C) 2007-2020 GoodData Corporation
var lodash_1 = require("lodash");
var Project_1 = require("../../model/Project");
var string_1 = require("../../utils/string");
var routes_1 = require("../../route/routes");
var objects_1 = require("../helpers/objects");
var ProjectBuilder_1 = require("./ProjectBuilder");
var UserBuilder_1 = require("./UserBuilder");
var DrillDefinitionBuilder_1 = require("./DrillDefinitionBuilder");
function getDashboardVisualizationWidgets(analyticalDashboard) {
    var widgets = analyticalDashboard.widgets
        ? analyticalDashboard.widgets.filter(function (widget) {
            return lodash_1.has(widget, "visualizationUri");
        })
        : [];
    return widgets;
}
function createVisualizationWidget(project, userId, visualizationWidget) {
    var identifier = objects_1.ensureIdentifier(visualizationWidget, string_1.webalize(visualizationWidget.title));
    var properties = lodash_1.get(visualizationWidget, "properties", "");
    var propertiesProp = properties.length > 0 ? { properties: properties } : {};
    return {
        content: __assign(__assign({ visualization: visualizationWidget.visualizationUri, dateDataSet: visualizationWidget.dateDataSetUri, ignoreDashboardFilters: visualizationWidget.ignoreDashboardFilters || [] }, (visualizationWidget.drills && {
            drills: DrillDefinitionBuilder_1.createDrillDefinitions(visualizationWidget.drills, project)
        })), propertiesProp),
        meta: {
            author: routes_1.profileUri(userId),
            category: "visualizationWidget",
            identifier: identifier,
            title: visualizationWidget.title,
            uri: routes_1.objectUri(Project_1.getProjectId(project), identifier)
        }
    };
}
exports.createVisualizationWidget = createVisualizationWidget;
function createVisualizationWidgets(schema) {
    var project = ProjectBuilder_1.createProject(schema);
    var users = UserBuilder_1.createUsers(schema);
    var visualizationWidgets = lodash_1.flatMap(schema.analyticalDashboards || [], function (analyticalDashboard) { return getDashboardVisualizationWidgets(analyticalDashboard) || []; });
    return visualizationWidgets.map(function (visualizationWidget) {
        return createVisualizationWidget(project, users[0].identifier, visualizationWidget);
    });
}
exports.createVisualizationWidgets = createVisualizationWidgets;
//# sourceMappingURL=VisualizationWidgetBuilder.js.map