// (C) 2019-2020 GoodData Corporation
import MeasureValueFilters from "../measureValueFilters";
import { ExecuteAFM as AFM } from "@gooddata/typings";

import { CachedResponse, isErrorResponse, IErrorResponse } from "../../executeAfm";
import * as oneColumnMeasureNoAttributeOneFilterExecution from "./fixtures/measureValueFilter/oneColumnMeasureNoAttributeOneFilterExecution.json";
import * as oneColumnMeasureOneColumnAttributeOneInvalidFilterExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneColumnAttributeOneInvalidFilterExecution.json";
import * as oneColumnMeasureOneColumnAttributeTwoNonUniqueFiltersExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneColumnAttributeTwoNonUniqueFiltersExecution.json";
import * as noMeasureOneColumnAttributeExecution from "./fixtures/measureValueFilter/noMeasureOneColumnAttributeExecution.json";
import * as noMeasureOneColumnAttributeResult from "./fixtures/measureValueFilter/noMeasureOneColumnAttributeResult.json";
import * as noMeasureValueFiltersExecution from "./fixtures/measureValueFilter/noMeasureValueFiltersExecution.json";
import * as noMeasureValueFiltersResult from "./fixtures/measureValueFilter/noMeasureValueFiltersResult.json";
import * as measureValueFilterWithUriExecution from "./fixtures/measureValueFilter/measureValueFilterWithUriExecution.json";
import * as measureValueFilterWithUriResult from "./fixtures/measureValueFilter/measureValueFilterWithUriResult.json";
import * as oneColumnMeasureOneColumnAttributeOneFilterExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneColumnAttributeOneFilterExecution.json";
import * as oneColumnMeasureOneColumnAttributeOneFilterResult from "./fixtures/measureValueFilter/oneColumnMeasureOneColumnAttributeOneFilterResult.json";
import * as oneColumnMeasureOneRowAttributeOneFilterExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneFilterExecution.json";
import * as oneColumnMeasureOneRowAttributeOneFilterResult from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneFilterResult.json";
import * as twoColumnMeasuresOneRowAttributeOneFilterExecution from "./fixtures/measureValueFilter/twoColumnMeasuresOneRowAttributeOneFilterExecution.json";
import * as twoColumnMeasuresOneRowAttributeOneFilterResult from "./fixtures/measureValueFilter/twoColumnMeasuresOneRowAttributeOneFilterResult.json";
import * as twoColumnMeasuresTwoRowAttributesOneColumnAttributeTwoFiltersExecution from "./fixtures/measureValueFilter/twoColumnMeasuresOneRowAttributeOneColumnAttributeTwoFiltersExecution.json";
import * as twoColumnMeasuresTwoRowAttributesOneColumnAttributeTwoFiltersResult from "./fixtures/measureValueFilter/twoColumnMeasuresOneRowAttributeOneColumnAttributeTwoFiltersResult.json";
import * as oneRowMeasureOneRowAttributeOneFilterExecution from "./fixtures/measureValueFilter/oneRowMeasureOneRowAttributeOneFilterExecution.json";
import * as oneRowMeasureOneRowAttributeOneFilterResult from "./fixtures/measureValueFilter/oneRowMeasureOneRowAttributeOneFilterResult.json";
import * as twoRowMeasuresOneRowAttributeOneFilterExecution from "./fixtures/measureValueFilter/twoRowMeasuresOneRowAttributeOneFilterExecution.json";
import * as twoRowMeasuresOneRowAttributeOneFilterResult from "./fixtures/measureValueFilter/twoRowMeasuresOneRowAttributeOneFilterResult.json";
import * as twoRowMeasuresOneRowAttributeTwoFiltersExecution from "./fixtures/measureValueFilter/twoRowMeasuresOneRowAttributeTwoFiltersExecution.json";
import * as twoRowMeasuresOneRowAttributeTwoFiltersResult from "./fixtures/measureValueFilter/twoRowMeasuresOneRowAttributeTwoFiltersResult.json";
import * as twoRowMeasuresTwoRowAttributesTwoFiltersExecution from "./fixtures/measureValueFilter/twoRowMeasuresTwoRowAttributesTwoFiltersExecution.json";
import * as twoRowMeasuresTwoRowAttributesTwoFiltersResult from "./fixtures/measureValueFilter/twoRowMeasuresTwoRowAttributesTwoFiltersResult.json";
import * as oneColumnMeasureOneRowAttributeOneFilterDataResult from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneFilterDataResult.json";
import * as oneRowMeasureOneStackedRowAttributeOneFilterExecution from "./fixtures/measureValueFilter/oneRowMeasureOneStackedRowAttributeOneFilterExecution.json";
import * as oneRowMeasureOneStackedRowAttributeOneFilterResult from "./fixtures/measureValueFilter/oneRowMeasureOneStackedRowAttributeOneFilterResult.json";
import * as oneColumnMeasureOneRowAttributeOneComparisonFilterWithoutTreatNullValuesAsExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneComparisonFilterWithoutTreatNullValuesAsExecution.json";
import * as oneColumnMeasureOneRowAttributeOneComparisonFilterWithTreatNullValuesAsExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneComparisonFilterWithTreatNullValuesAsExecution.json";
import * as oneColumnMeasureOneRowAttributeOneRangeFilterWithoutTreatNullValuesAsExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneRangeFilterWithoutTreatNullValuesAsExecution.json";
import * as oneColumnMeasureOneRowAttributeOneRangeFilterWithTreatNullValuesAsExecution from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneRangeFilterWithTreatNullValuesAsExecution.json";
import * as oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult from "./fixtures/measureValueFilter/oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult.json";
import { IRandomDataResult } from "../dataResult";
import IExecution = AFM.IExecution;

describe("measureValueFilters", () => {
    describe("filterExecutionResult", () => {
        it("should return result without change if numeric error response is provided", () => {
            const errorResponse = 500;
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                errorResponse,
            );
            expect(filteredResult).toEqual(errorResponse);
        });

        it("should return result without change if error response is provided", () => {
            const errorResponse: IErrorResponse = {
                error: {
                    statusCode: 400,
                    message: "some error",
                },
            };
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                errorResponse,
            );
            expect(filteredResult).toEqual(errorResponse);
        });

        it("should return result without change if there are no measure", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                noMeasureOneColumnAttributeResult as CachedResponse,
            );
            expect(filteredResult).toEqual(noMeasureOneColumnAttributeResult);
        });

        it("should return error result if there are no attributes in execution", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                oneColumnMeasureNoAttributeOneFilterExecution as IExecution,
                noMeasureOneColumnAttributeResult as CachedResponse,
            );
            expect(isErrorResponse(filteredResult)).toBeTruthy();
            expect((filteredResult as IErrorResponse).error.statusCode).toEqual(400);
        });

        it("should return error result if measure value filter references invalid measure", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                oneColumnMeasureOneColumnAttributeOneInvalidFilterExecution as IExecution,
                noMeasureOneColumnAttributeResult as CachedResponse,
            );
            expect(isErrorResponse(filteredResult)).toBeTruthy();
            expect((filteredResult as IErrorResponse).error.statusCode).toEqual(400);
        });

        it("should return error result if multiple measure value filters reference the same measure", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                oneColumnMeasureOneColumnAttributeTwoNonUniqueFiltersExecution as IExecution,
                noMeasureOneColumnAttributeResult as CachedResponse,
            );
            expect(isErrorResponse(filteredResult)).toBeTruthy();
            expect((filteredResult as IErrorResponse).error.statusCode).toEqual(400);
        });

        it("should return result without change if there are no measure value filters", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                noMeasureValueFiltersExecution as IExecution,
                noMeasureValueFiltersResult as CachedResponse,
            );
            expect(filteredResult).toEqual(noMeasureValueFiltersResult);
        });

        it("should return result without change if there is measure value filters with URI", () => {
            const filteredResult = MeasureValueFilters.filterExecutionResult(
                measureValueFilterWithUriExecution as IExecution,
                measureValueFilterWithUriResult as CachedResponse,
            );
            expect(filteredResult).toEqual(noMeasureValueFiltersResult);
        });

        describe("measures in columns", () => {
            it("should keep only expected values for one measure and one row attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneFilterExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for one measure and one column attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneColumnAttributeOneFilterExecution as IExecution,
                    oneColumnMeasureOneColumnAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    twoColumnMeasuresOneRowAttributeOneFilterExecution as IExecution,
                    twoColumnMeasuresOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with two filters", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    twoColumnMeasuresTwoRowAttributesOneColumnAttributeTwoFiltersExecution as IExecution,
                    twoColumnMeasuresTwoRowAttributesOneColumnAttributeTwoFiltersResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
        });

        describe("measures in rows", () => {
            it("should keep only expected values for one measure and one stacked row attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneRowMeasureOneStackedRowAttributeOneFilterExecution as IExecution,
                    oneRowMeasureOneStackedRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for one measure and one row attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneRowMeasureOneRowAttributeOneFilterExecution as IExecution,
                    oneRowMeasureOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with one filter", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    twoRowMeasuresOneRowAttributeOneFilterExecution as IExecution,
                    twoRowMeasuresOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with two filters", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    twoRowMeasuresOneRowAttributeTwoFiltersExecution as IExecution,
                    twoRowMeasuresOneRowAttributeTwoFiltersResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and two row attribute with two filters", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    twoRowMeasuresTwoRowAttributesTwoFiltersExecution as IExecution,
                    twoRowMeasuresTwoRowAttributesTwoFiltersResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
        });

        describe("treatNullValuesAs", () => {
            it("should not return null values if treatNullValuesAs is not set for comparison condition", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneComparisonFilterWithoutTreatNullValuesAsExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should treat null values as 0 if treatNullValuesAs is 0 for comparison condition", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneComparisonFilterWithTreatNullValuesAsExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
            it("should not return null values if treatNullValuesAs is not set for range condition", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneRangeFilterWithoutTreatNullValuesAsExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should treat null values as 0 if treatNullValuesAs is 0 for range condition", () => {
                const filteredResult = MeasureValueFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneRangeFilterWithTreatNullValuesAsExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterWithNullValuesResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
        });
    });

    describe("filterDataResult", () => {
        it("should return error result if some validation rule fails", () => {
            const filteredResult = MeasureValueFilters.filterDataResult(
                oneColumnMeasureNoAttributeOneFilterExecution as IExecution,
                oneColumnMeasureOneRowAttributeOneFilterDataResult as IRandomDataResult,
            );
            expect(isErrorResponse(filteredResult)).toBeTruthy();
            expect((filteredResult as IErrorResponse).error.statusCode).toEqual(400);
        });

        it("should keep only expected values for one column measure, one row attribute, and one filter ", () => {
            const filteredResult = MeasureValueFilters.filterDataResult(
                oneColumnMeasureOneRowAttributeOneFilterExecution as IExecution,
                oneColumnMeasureOneRowAttributeOneFilterDataResult as IRandomDataResult,
            );
            expect(filteredResult).toMatchSnapshot();
        });
    });
});
