// (C) 2019-2020 GoodData Corporation
import RankingFilters from "../rankingFilters";
import { ExecuteAFM as AFM } from "@gooddata/typings";

import { CachedResponse, IErrorResponse } from "../../executeAfm";
import * as noMeasureOneColumnAttributeExecution from "./fixtures/rankingFilter/noMeasureOneColumnAttributeExecution.json";
import * as noMeasureOneColumnAttributeResult from "./fixtures/rankingFilter/noMeasureOneColumnAttributeResult.json";
import * as noRankingFiltersExecution from "./fixtures/rankingFilter/noRankingFiltersExecution.json";
import * as noRankingFiltersResult from "./fixtures/rankingFilter/noRankingFiltersResult.json";
import * as oneColumnMeasureOneColumnAttributeOneFilterExecution from "./fixtures/rankingFilter/oneColumnMeasureOneColumnAttributeOneFilterExecution.json";
import * as oneColumnMeasureOneColumnAttributeOneFilterResult from "./fixtures/rankingFilter/oneColumnMeasureOneColumnAttributeOneFilterResult.json";
import * as oneColumnMeasureOneRowAttributeOneFilterExecution from "./fixtures/rankingFilter/oneColumnMeasureOneRowAttributeOneFilterExecution.json";
import * as oneColumnMeasureOneRowAttributeOneFilterResult from "./fixtures/rankingFilter/oneColumnMeasureOneRowAttributeOneFilterResult.json";
import * as twoColumnMeasuresOneRowAttributeOneFilterExecution from "./fixtures/rankingFilter/twoColumnMeasuresOneRowAttributeOneFilterExecution.json";
import * as twoColumnMeasuresOneRowAttributeOneFilterResult from "./fixtures/rankingFilter/twoColumnMeasuresOneRowAttributeOneFilterResult.json";
import * as oneRowMeasureOneRowAttributeOneFilterExecution from "./fixtures/rankingFilter/oneRowMeasureOneRowAttributeOneFilterExecution.json";
import * as oneRowMeasureOneRowAttributeOneFilterResult from "./fixtures/rankingFilter/oneRowMeasureOneRowAttributeOneFilterResult.json";
import * as twoRowMeasuresOneRowAttributeOneFilterExecution from "./fixtures/rankingFilter/twoRowMeasuresOneRowAttributeOneFilterExecution.json";
import * as twoRowMeasuresOneRowAttributeOneFilterResult from "./fixtures/rankingFilter/twoRowMeasuresOneRowAttributeOneFilterResult.json";
import * as oneColumnMeasureOneRowAttributeOneFilterDataResult from "./fixtures/rankingFilter/oneColumnMeasureOneRowAttributeOneFilterDataResult.json";
import * as oneRowMeasureOneStackedRowAttributeOneFilterExecution from "./fixtures/rankingFilter/oneRowMeasureOneStackedRowAttributeOneFilterExecution.json";
import * as oneRowMeasureOneStackedRowAttributeOneFilterResult from "./fixtures/rankingFilter/oneRowMeasureOneStackedRowAttributeOneFilterResult.json";
import { IRandomDataResult } from "../dataResult";
import IExecution = AFM.IExecution;

describe("rankingFilters", () => {
    describe("filterExecutionResult", () => {
        it("should return result without change if numeric error response is provided", () => {
            const errorResponse = 500;
            const filteredResult = RankingFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                errorResponse,
            );
            expect(filteredResult).toEqual(errorResponse);
        });

        it("should return result without change if error response is provided", () => {
            const errorResponse: IErrorResponse = {
                error: {
                    statusCode: 400,
                    message: "some error",
                },
            };
            const filteredResult = RankingFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                errorResponse,
            );
            expect(filteredResult).toEqual(errorResponse);
        });

        it("should return result without change if there are no measure", () => {
            const filteredResult = RankingFilters.filterExecutionResult(
                noMeasureOneColumnAttributeExecution as IExecution,
                noMeasureOneColumnAttributeResult as CachedResponse,
            );
            expect(filteredResult).toEqual(noMeasureOneColumnAttributeResult);
        });

        it("should return result without change if there are no ranking filters", () => {
            const filteredResult = RankingFilters.filterExecutionResult(
                noRankingFiltersExecution as IExecution,
                noRankingFiltersResult as CachedResponse,
            );
            expect(filteredResult).toEqual(noRankingFiltersResult);
        });

        describe("measures in columns", () => {
            it("should keep only expected values for one measure and one row attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    oneColumnMeasureOneRowAttributeOneFilterExecution as IExecution,
                    oneColumnMeasureOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for one measure and one column attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    oneColumnMeasureOneColumnAttributeOneFilterExecution as IExecution,
                    oneColumnMeasureOneColumnAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    twoColumnMeasuresOneRowAttributeOneFilterExecution as IExecution,
                    twoColumnMeasuresOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
        });

        describe("measures in rows", () => {
            it("should keep only expected values for one measure and one stacked row attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    oneRowMeasureOneStackedRowAttributeOneFilterExecution as IExecution,
                    oneRowMeasureOneStackedRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for one measure and one row attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    oneRowMeasureOneRowAttributeOneFilterExecution as IExecution,
                    oneRowMeasureOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });

            it("should keep only expected values for two measures and one row attribute with one filter", () => {
                const filteredResult = RankingFilters.filterExecutionResult(
                    twoRowMeasuresOneRowAttributeOneFilterExecution as IExecution,
                    twoRowMeasuresOneRowAttributeOneFilterResult as CachedResponse,
                );
                expect(filteredResult).toMatchSnapshot();
            });
        });
    });

    describe("filterDataResult", () => {
        it("should keep only expected values for one column measure, one row attribute, and one filter ", () => {
            const filteredResult = RankingFilters.filterDataResult(
                oneColumnMeasureOneRowAttributeOneFilterExecution as IExecution,
                oneColumnMeasureOneRowAttributeOneFilterDataResult as IRandomDataResult,
            );
            expect(filteredResult).toMatchSnapshot();
        });
    });
});
