// (C) 2007-2019 GoodData Corporation
import { Application } from "express";
import { IMockProject } from "../../model/MockProject";
import * as HttpStatusCodes from "http-status-codes";
import { IStyleSettings } from "../../model/StyleSetting";

interface IUserProject {
    userProject: {
        links: {
            projectUser: string;
            self: string;
            projectDescription: string;
            projectState: string;
            projectTitle: string;
            userState: string;
        };
    };
}

interface IProjectsResponse {
    userProjects: {
        items: IUserProject[];
        paging: {
            count: number;
            limit: number;
            offset: number;
            totalCount: number;
        };
    };
}

interface IStyleSettingsResponse {
    styleSettings: IStyleSettings;
}

export const projects = {
    register(app: Application, project: IMockProject) {
        app.get("/gdc/internal/projects", (_req, res) => {
            const body: IProjectsResponse = {
                userProjects: {
                    items: [],
                    paging: {
                        count: 0,
                        limit: 100,
                        offset: 0,
                        totalCount: 0,
                    },
                },
            };

            return res.status(HttpStatusCodes.OK).send(body);
        });

        app.get(`/gdc/projects/${project.project.meta.identifier}/styleSettings`, (_req, res) => {
            const body: IStyleSettingsResponse = {
                styleSettings: project.styleSettings,
            };

            res.status(HttpStatusCodes.OK).send(body);
        });

        return app;
    },
};
