// (C) 2007-2019 GoodData Corporation
import { csv, ICsvDataSetsResponse } from "../csv";
import * as request from "supertest";
import { createEndpoint } from "../../../utils/tests";
import { ISchema } from "../../../schema/model/Schema";

it("should return list of datasets", () => {
    const schema: ISchema = {
        project: {
            title: "Foo",
        },
        groups: [
            {
                meta: {
                    identifier: "ignored",
                    type: "production",
                },
            },
            {
                meta: {
                    identifier: "mydataset",
                    type: "csv",
                },
            },
        ],
    };

    const app = createEndpoint(csv, schema);

    return request(app)
        .get("/gdc/dataload/internal/projects/mockproject/csv/datasets")
        .then(res => {
            const expectedBody: ICsvDataSetsResponse = {
                datasets: {
                    items: [
                        {
                            dataset: {
                                name: "mydataset",
                                datasetId: "mydataset",
                                datasetLoadStatus: "OK",
                                firstSuccessfulUpdate: {
                                    created: "2015-12-18T10:20:35.532Z",
                                    owner: {
                                        fullName: "John Doe",
                                        login: "john.doe@gooddata.com",
                                        profileUri: "/gdc/account/profile/john-doe",
                                    },
                                },
                                loadedRowCount: 1000,
                            },
                        },
                    ],
                },
            };

            expect(res.body).toEqual(expectedBody);
        });
});
