// (C) 2019-2020 GoodData Corporation
import { OK } from "http-status-codes";
import { Application } from "express";
import { IMockProject } from "../../model/MockProject";
import { getBootstrap } from "./bootstrap";

export function getProfileUri(bootstrap: any) {
    const {
        bootstrapResource: {
            accountSetting: { links: self },
        },
    } = bootstrap;
    return self.self;
}

export const userConfigs = {
    register(app: Application, project: IMockProject) {
        const bootstrap = getBootstrap(project);
        app.get(`${getProfileUri(bootstrap)}/config`, (_req, res) => {
            res.status(OK).json({
                settings: {
                    items: project.userConfigs,
                },
            });
        });

        const separatorsUri = `${getProfileUri(bootstrap)}/settings/separators`;

        app.get(separatorsUri, (_req, res) => {
            res.status(OK).json({
                separators: {
                    thousand: ",",
                    decimal: ".",
                    links: {
                        self: separatorsUri,
                    },
                },
            });
        });

        return app;
    },
};
